#!/usr/bin/env python
"""Test watchdog restart"""

############################################################################
#
#   Crasher
#
#   This script runs for awhile then dies to test the watchdog.
#   Assumes that the ServerMonitor/Watchdog is running.
#
#   History:
#
#   2002-12-16  Todd Valentic
#               Initial implementation
#
#   2005-01-09  Todd Valentic
#               Use DeltaTime for lifetime.
#
#   2022-11-04  Todd Valentic
#               Update for python3
#
############################################################################

import sys

from datatransport import ProcessClient


class Crasher(ProcessClient):
    """Process that unexpectedly dies"""

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

    def main(self):
        """Main loop"""

        lifetime = self.config.get_rate("lifetime", 10)

        while self.wait(lifetime):
            self.log.info("About to crash!")
            self.log.info("badness = %f", 1 / 0)


if __name__ == "__main__":
    Crasher(sys.argv).run()



from .archivegroups     import ArchiveGroups
from .directoryservice  import DirectoryService
from .diskmonitor       import DiskMonitor
from .filewatch         import FileWatch
from .filepost          import FilePost
from .groupcontrol      import GroupControl
from .InstrumentStatus	import InstrumentStatus
from .NewsgroupMonitor  import NewsgroupMonitor
from .newsgateway       import NewsGateway 
from .PlotTool          import PlotTool
from .postdatafiles     import PostDataFiles
from .realtimefeed      import RealTimeFeed
from .resourcemonitor	import ResourceMonitor
from .scheduler         import Scheduler, SchedulerEvent
from .syncpoller        import SyncPoller
from .watchurl	        import WatchURL



#!/usr/bin/env python3
"""Resource monitor"""

# pylint: disable=broad-exception-caught

##########################################################################
#
#   Resource Monitor
#
#   Collect and post a snapshot of the computer health:
#
#       - disk usage
#       - memory usage
#       - processor load
#       - uptime
#       - network usage
#
#   1.0.0   2004-02-01  Todd Valentic
#           Initial implementation.
#
#   1.0.1   2004-02-17  Todd Valentic
#           Handle rollover in tx/rx rate computation.
#           Tx/Rx rates were swapped.
#
#   1.0.2   2004-08-24  Todd Valentic
#           Changed memory section to just pass on values
#               found in /proc/meminfo (the summary format
#               was dropped in the 2.6.X kernels).
#
#   1.0.3   2004-09-19  Todd Valentic
#           Added swap statistics from /proc/swaps
#
#   1.0.4   2004-012-27 Todd Valentic
#           Convert from mx.DateTime to datetime
#
#   1.0.5   2008-11-10  Todd Valentic
#           Use SafeConfigParser
#
#   1.0.6   2008-12-08  Todd Valentic
#           Make sure all set() calls use strings
#
#   1.0.7   2009-11-24  Todd Valentic
#           Use currentTime()
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#               Python3 updates:
#                   ConfigParser -> configparser
#                   StringIO -> io.StringIO
#               Use get_rate()
#               Use NewsPoster
#
#   2023-08-11  Todd Valentic
#               Updated for transport3 / python3
#
##########################################################################

import io
import os
import sys

from pathlib import Path

from datatransport import ProcessClient
from datatransport import NewsPoster
import sapphire_config as sapphire


class ResourceMonitor(ProcessClient):
    """Resource Monitor"""

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        self.news_poster = NewsPoster(self)
        self.rate = self.config.get_rate("rate", 600)

        self.lasttx = {}
        self.lastrx = {}

    def read_proc(self, path):
        """Read data from /proc path"""

        return Path("/proc", path).read_text('utf-8').split("\n")

    def update_mounts(self, stats):
        """Update status of file mounts"""

        mounts = self.read_proc("mounts")
        mounts = [mount.split()[0:4] for mount in mounts if mount]

        paths = []

        for device, path, fstype, access in mounts:
            try:
                info = os.statvfs(path)
            except OSError:
                continue

            if device == "none":
                continue

            if info.f_blocks == 0:
                continue

            if stats.has_section(path):
                continue

            paths.append(path)

            totalbytes = info.f_blocks * info.f_bsize
            freebytes = info.f_bavail * info.f_bsize
            reserved = info.f_bfree * info.f_bsize - freebytes
            totalavail = totalbytes - reserved
            usedbytes = totalavail - freebytes
            usedpct = usedbytes / float(totalavail) * 100

            stats.add_section(path)
            stats.set(path, "device", device)
            stats.set(path, "fstype", fstype)
            stats.set(path, "access", access)
            stats.set(path, "totalbytes", str(totalavail))
            stats.set(path, "freebytes", str(freebytes))
            stats.set(path, "usedbytes", str(usedbytes))
            stats.set(path, "usedpct", str(usedpct))

        stats.set("System", "mounts", " ".join(paths))

    def update_memory(self, stats):
        """Update memory usage stats"""

        lines = self.read_proc("meminfo")

        if "total:" in lines[0]:  # old style format
            lines = lines[3:]

        section = "Memory"
        stats.add_section(section)

        for line in lines:
            try:
                key, value = line.split(":")
                value = int(value.split()[0]) * 1024
                stats.set(section, key, str(value))
            except Exception:
                pass

    def update_load(self, stats):
        """Update load stats"""

        info = self.read_proc("loadavg")
        load = info[0].split()

        section = "Load"

        stats.add_section(section)
        stats.set(section, "1min", load[0])
        stats.set(section, "5min", load[1])
        stats.set(section, "15min", load[2])

    def update_swaps(self, stats):
        """Update swap usage"""

        section = "Swaps"
        stats.add_section(section)

        info = self.read_proc("swaps")

        swaps = []

        for line in info[1:-1]:
            if not line:
                continue
            try:
                dev, fstype, size, used, priority = line.split()
            except ValueError:
                continue
            swaps.append(dev)
            stats.set(section, dev + ".type", fstype)
            stats.set(section, dev + ".size", str(int(size) * 1024))
            stats.set(section, dev + ".used", str(int(used) * 1024))
            stats.set(section, dev + ".priority", priority)

        stats.set(section, "mounts", " ".join(swaps))

    def update_uptime(self, stats):
        """Update uptime stats"""

        info = self.read_proc("uptime")
        secs = float(info[0].split()[0])

        section = "Uptime"
        stats.add_section(section)
        stats.set(section, "seconds", str(secs))

    def compute_rate(self, prevbytes, prevtime, curbytes, curtime):
        """Update network data rates"""

        if prevbytes > curbytes:
            # Counter rollover
            deltabytes = 2**32 - prevbytes + curbytes
        else:
            deltabytes = curbytes - prevbytes
        return deltabytes / (curtime - prevtime).seconds

    def update_network(self, stats):
        """Update networks"""

        info = self.read_proc("net/dev")[2:-1]

        section = "Network"
        stats.add_section(section)

        devs = []
        now = self.now()

        for device in info:
            name, data = device.split(":")
            name = name.strip()
            data = data.split()

            if name in self.lasttx:
                txbytes, txtime = self.lasttx[name]
                rxbytes, rxtime = self.lastrx[name]
                txrate = self.compute_rate(txbytes, txtime, float(data[8]), now)
                rxrate = self.compute_rate(rxbytes, rxtime, float(data[0]), now)
            else:
                txrate = 0
                rxrate = 0

            stats.set(section, name + ".rx.rate", str(rxrate))
            stats.set(section, name + ".rx.bytes", data[0])
            stats.set(section, name + ".rx.packets", data[1])
            stats.set(section, name + ".rx.errs", data[2])
            stats.set(section, name + ".rx.drop", data[3])

            stats.set(section, name + ".tx.rate", str(txrate))
            stats.set(section, name + ".tx.bytes", data[8])
            stats.set(section, name + ".tx.packets", data[9])
            stats.set(section, name + ".tx.errs", data[10])
            stats.set(section, name + ".tx.drop", data[11])

            self.lasttx[name] = (float(data[8]), now)
            self.lastrx[name] = (float(data[0]), now)

            devs.append(name)

        stats.set(section, "devices", " ".join(devs))

    def process(self):
        """Gather new status snapshot"""

        timestamp = self.now()

        stats = sapphire.Parser()
        stats.add_section("System")
        stats.set("System", "timestamp", str(timestamp))

        self.update_mounts(stats)
        self.update_memory(stats)
        self.update_load(stats)
        self.update_uptime(stats)
        self.update_network(stats)
        self.update_swaps(stats)

        buffer = io.StringIO()
        stats.write(buffer)

        self.news_poster.post_text(buffer.getvalue(), date=timestamp)

        self.log.info("Posting snapshot")

    def main(self):
        while self.wait(self.rate):
            try:
                self.process()
            except Exception:  # pylint: disable=broad-exception-caught
                self.log.exception("Error updating snapshot")


def main():
    """Script entry point"""
    ResourceMonitor(sys.argv).run()



#!/usr/bin/env python
"""Transport Logger"""

##########################################################################
#
#   Create log handlers
#
#   2022-10-07  Todd Valentic
#               Add missing comment block header
#               Use get_* methods
#
##########################################################################

import logging

from logging.handlers import RotatingFileHandler
from logging.handlers import SocketHandler


from .utilities import make_path


def _setup_log_socket_handler(config, formatter):
    """Add a socket log handler"""

    host = config.get("log.socket.host", "localhost")
    port = config.get_int("log.socket.port", 9020)

    socket_handler = SocketHandler(host, port)
    socket_handler.setFormatter(formatter)

    return socket_handler


def _setup_log_file_handler(config, formatter):
    """Add a rotating file log handler"""

    filename = config.get("log.file")
    maxbytes = config.get_bytes("log.maxbytes", "100kb")
    num_backup = config.get_int("log.backupcount", 3)

    make_path(filename)

    rotating_handler = RotatingFileHandler(filename, "a", maxbytes, num_backup)
    rotating_handler.setFormatter(formatter)

    return rotating_handler


def _setup_log_formatter():
    """Initialize a log formatter"""

    msgfmt = "[%(asctime)s.%(msecs)03d %(levelname)7s] %(name)s: %(message)s"
    datefmt = "%Y-%m-%d %H:%M:%S"

    return logging.Formatter(msgfmt, datefmt)


def create_transport_logger(config, name):
    """Create a logger for data transport processes"""

    level = config.get("log.level", "info")
    formatter = _setup_log_formatter()

    logger = logging.getLogger(name)

    if config.get_boolean("log.file.enable", True):
        logger.addHandler(_setup_log_file_handler(config, formatter))

    if config.get_boolean("log.socket.enable", True):
        logger.addHandler(_setup_log_socket_handler(config, formatter))

    if level == "error":
        logger.setLevel(logging.ERROR)
    if level == "warning":
        logger.setLevel(logging.WARNING)
    elif level == "info":
        logger.setLevel(logging.INFO)
    elif level == "debug":
        logger.setLevel(logging.DEBUG)

    return logger



#!/usr/bin/env python
"""Transport Server"""

##########################################################################
#
#   Transport Server
#
#   2005-11-08  Todd Valentic
#               Only list process groups with clients in listgroups().
#
#   2006-09-15  Todd Valentic
#               Check if process is alive in expire().
#
#   2007-03-11  Todd Valentic
#               Added ident() method to API to make callable from
#                   directory services.
#               Register introspection services.
#
#   2008-11-10  Todd Valentic
#               Pass sever log file to process groups to record any
#                   initial failures.
#
#   2009-05-11  Todd Valentic
#               Since we now require Python 2.4 or greater, we don't
#                   need to import the sets module anymore.
#
#   2010-01-19  Todd Valentic
#               Add group priority start/stop
#
#   2010-01-25  Todd Valentic
#               Rework stopping groups. Stop the clients in each group
#                   and wait until they are stopped before continuing on.
#                   One significant change is that stop() will block.
#
#   2010-05-07  Todd Valentic
#               Make sure to return value in addgroup()
#
#   2016-07-10  Todd Valentic
#               Make a threaded server - note that there are no saftey
#                   checks for concurent access! Made this change to
#                   allow clients to log out (otherwise stop blocked).
#
#   2016-11-08  Todd Valentic
#               Make try..finally compatible with python 2.4
#
#   2023-10-07  Todd Valentic
#               Reorder imports
#               Python3: xmlrpc.server
#               Python3: socketserver
#               Use new get_* methods
#
##########################################################################

import os
import pathlib
import resource

from threading import Thread
from socketserver import ThreadingMixIn
from xmlrpc.server import SimpleXMLRPCServer

from .transportlogger import create_transport_logger
from . import ProcessGroup
from . import TransportConfig

# pylint: disable=too-many-public-methods


class TransportServer(Thread, ThreadingMixIn, SimpleXMLRPCServer):
    """Transport Server"""

    def __init__(self, queue):
        Thread.__init__(self)
        self.daemon = True

        self.config = TransportConfig()["TransportServer"]
        self.log = create_transport_logger(self.config, "TransportServer")
        self.autostart = self.config.get_boolean("autostart", False)

        port = self.config.get_int("port", 8081)
        self.allow_reuse_address = 1
        self.request_queue_size = 100
        SimpleXMLRPCServer.__init__(self, ("", port), logRequests=False)
        self.socket.settimeout(1)
        self.timeout = 1

        self.setup_xmlrpc()
        self.setup_environ()

        self.queue = queue
        self.running = True

        self.groups = {}
        self.pidlist = {}

        self.log.info("---------- START ----------")

        self.loadgroups()

    def setup_xmlrpc(self):
        """Setup XMLRPC API"""

        self.register_introspection_functions()

        self.register_function(self.my_ident, "ident")
        self.register_function(self.status)
        self.register_function(self.stop)

        self.register_function(self.listgroups)
        self.register_function(self.reloadgroups)

        self.register_function(self.addgroup)
        self.register_function(self.removegroup)
        self.register_function(self.startgroup)
        self.register_function(self.stopgroup)
        self.register_function(self.reloadgroup)

        self.register_function(self.startclient)
        self.register_function(self.stopclient)
        self.register_function(self.listclients)
        self.register_function(self.loginclient)
        self.register_function(self.logoutclient)

    def setup_environ(self):
        """Run process groups in a controlled, pristine environment"""

        keep = ["PYTHONPATH", "PATH", "DATA_TRANSPORT_PATH"]

        environ = {}

        for key, value in os.environ.items():
            if key in keep:
                environ[key] = value

        environ["HOME"] = self.config.get("path.base")

        os.environ = environ

    def my_ident(self):
        """Return ID string"""

        return "Data transport server"

    def checkgroup(self, name, action):
        """Check if process group exists"""

        if name not in self.groups:
            self.log.error("Request to %s unknown group: %s", action, name)
            raise NameError(name)

    def findgroups(self, path):
        """Find all process groups in the file heirarchy"""

        groups = []

        for root, _dirs, files in os.walk(path, followlinks=True):
            configname = os.path.basename(root) + ".conf"
            if configname in files:
                groups.append(root.replace(path, "", 1)[1:])

        return groups

    def loadgroups(self):
        """Load group from config file"""

        path = self.config.get("path.groups")
        grouplist = self.findgroups(path)

        stalegroups = set(self.groups).difference(grouplist)
        newgroups = set(grouplist).difference(self.groups)
        curgroups = set(grouplist).intersection(self.groups)

        self.stopgroups(stalegroups)

        for name in stalegroups:
            del self.groups[name]

        for name in newgroups:
            self.creategroup(name)

        if self.autostart:
            # filter out new groups that failed being created
            self.startgroups(newgroups.intersection(self.groups))

        for name in curgroups:
            self.reloadgroup(name)

    def stopgroups(self, names):
        """Stop process groups according to priority"""

        priority_list = []

        for name in names:
            group = self.groups[name]
            priority_list.append((group.stop_priority, name))

        for _level, name in sorted(priority_list):
            self.stopgroup(name)

    def startgroups(self, names):
        """Start process groups according to priority"""

        priority_list = []

        for name in names:
            group = self.groups[name]
            if group.autostart:
                priority_list.append((group.start_priority, name))

        for _level, name in sorted(priority_list):
            self.startgroup(name)

    def removegroup(self, name):
        """Remove a process group"""

        self.checkgroup(name, "remove")

        self.log.info("Removing %s", name)
        self.groups[name].stop()
        del self.groups[name]

        return True

    def creategroup(self, name):
        """Create a new process group"""

        basedir = self.config.get("path.groups")

        if not os.path.isdir(os.path.join(basedir, name)):
            self.log.error("Request to add unknown process group: %s", name)
            raise NameError

        if name in self.groups:
            self.log.error("Cannot add group, already exists: %s", name)
            return False

        try:
            self.groups[name] = ProcessGroup(name, self.queue, self.log)
            self.log.info("Adding %s", name)
        except BaseException:  # pylint: disable=broad-except
            self.log.exception("Problem loading %s", name)
            return False

        return True

    def addgroup(self, name):
        """Add a process group (create, start)"""

        self.creategroup(name)
        self.startgroup(name)
        return True

    def reloadgroup(self, name):
        """Reload process group configuration"""

        self.checkgroup(name, "reload")
        self.log.info("Reloading %s", name)
        self.groups[name].reload()
        return True

    def reloadgroups(self):
        """Reload all process group configurations"""

        self.config = TransportConfig()["TransportServer"]
        self.loadgroups()
        return True

    def startgroup(self, name):
        """Start a process group running"""

        self.checkgroup(name, "start")
        group = self.groups[name]
        self.log.info("Starting [%d] %s", group.start_priority, name)
        group.start()
        return True

    def stopgroup(self, name):
        """Stop a process group"""

        self.checkgroup(name, "stop")
        group = self.groups[name]
        self.log.info("Stopping [%2d] %s", group.stop_priority, name)
        return group.stop()

    def listgroups(self):
        """List process groups"""

        results = {}

        for name, group in self.groups.items():
            if group.list_clients():
                results[name] = group.grouplabel

        return results

    def startclient(self, group, client, args=""):
        """Start a process group client"""

        self.checkgroup(group, "start client")
        self.groups[group].start_client(client, args)
        return True

    def stopclient(self, group, client):
        """Stop a process group client"""

        self.checkgroup(group, "stop client")
        self.groups[group].stop_client(client)
        return True

    def loginclient(self, group, client, pid):
        """Login a running client"""

        self.checkgroup(group, "login client")
        self.groups[group].login(client, pid)
        return True

    def logoutclient(self, group, client, pid):
        """Logout a client"""

        self.checkgroup(group, "logout client")
        self.groups[group].logout(client, pid)

        if pid in self.pidlist:
            del self.pidlist[pid]

        return True

    def listclients(self, group):
        """List all of the clients in a process group"""

        self.checkgroup(group, "listclients")
        return self.groups[group].list_clients()

    def stop(self):
        """Stop the server"""

        self.stopgroups(self.groups.keys())
        self.running = False
        return True

    def status(self):
        """Indicate we are alive"""

        pid = os.getpid()
        proc = pathlib.Path('/proc', str(pid), 'fd')
        numfiles = len(list(proc.iterdir()))
        numclients = sum(len(g.clients) for g in self.groups.values())
        numrunning = 0

        for group in self.groups.values():
            for client in group.clients.values():
                if client.pid:
                    numrunning += 1

        statm = pathlib.Path('/proc', str(pid), 'statm')
        vmrss = int(statm.read_text("UTF-8").split()[1])
        memusage = vmrss * resource.getpagesize()

        results = {}
        results["pid"] = os.getpid()
        results["open_files"] = numfiles
        results["memory"] = memusage
        results["groups"] = len(self.groups)
        results["clients"] = numclients
        results["running"] = numrunning

        return results

    def run(self):
        """Main loop"""

        self.log.info("Starting server thread")

        try:
            self.serve_forever()
        except BaseException as err:  # pylint: disable=broad-except
            self.log.exception("Problem: (%s) %s", type(err), str(err))
        finally:
            self.server_close()

        self.log.info("========== STOP  ==========")



#!/usr/bin/env python2

############################################################################
#
#	Spawner	
#
#	This script runs child process at specified intervals 
#
#	History:
#	
#	1.0.0 	2002-12-19	Todd Valentic	
#			Initial implementation
#
############################################################################

from 	Transport	import ProcessClient

import	sys
import 	commands

class Spawner (ProcessClient):

	def __init__(self,argv):
		ProcessClient.__init__(self,argv)

	def run(self):

		rate	= self.getint('spawn.rate',10)
		cmd		= self.get('spawn.command','pwd')

		while self.running:

			self.log.info('Running command: %s' % cmd)

			try:
				status,output = commands.getstatusoutput(cmd)	
			except:
				self.log.exception('Problem starting child')
				self.abort('Exiting')

			self.log.info('  Status=%d' % status)
			self.log.info('  Output=%s' % output)

			self.wait(rate)

if __name__ == '__main__':
	Spawner(sys.argv).run()




#!/usr/bin/env python3
"""File polling example"""

##########################################################################
#
#   Polling example (file attachment)
#
#   Periodically poll for messages from a news group
#
#   2022-10-10  Todd Valentic
#               Initial implementation
#
##########################################################################

import sys

from datatransport import ProcessClient
from datatransport import NewsPoller
from datatransport import newstool
from datatransport.utilities import remove_file


class Client(ProcessClient):
    """Process client"""

    def __init__(self, args):
        ProcessClient.__init__(self, args)

        self.poller = NewsPoller(self, callback=self.process)
        self.main = self.poller.main

        self.log.info("Poll for messages")

    def process(self, message):
        """Process handler"""

        filenames = newstool.save_files(message)
        self.log.info("Received files: %s", [str(f) for f in filenames])

        remove_file(filenames)


if __name__ == "__main__":
    Client(sys.argv).run()




import pytest
import zoneinfo

from datetime import datetime, timedelta, timezone
from datatransport.utilities import datefunc 

@pytest.mark.parametrize('timedelta_str, expected_result', [
    ('0', timedelta()),
    ('9', timedelta(seconds=9)),
    ('5s', timedelta(seconds=5)),
    ('2:23', timedelta(minutes=2, seconds=23)),
    ('43:23', timedelta(minutes=43, seconds=23)),
    ('23:32:43', timedelta(hours=23, minutes=32, seconds=43)),
    ('1 day', timedelta(days=1)),
    ('2 days', timedelta(days=2)),
    ('2.5 days', timedelta(days=2, hours=12)),
    ('1 days 12:34:45', timedelta(days=1, hours=12, minutes=34, seconds=45)),
    ])

def test_datefunc_parse_timedelta(timedelta_str, expected_result):
    result = datefunc.parse_timedelta(timedelta_str) 
    assert result == expected_result 

@pytest.mark.parametrize('bad_timedelta_str', [
    '32:2',
    '1 days 2',
    '1 days 23:23',
    '',
    'abc'
    ])

def test_datefunc_parse_timedelta_failures(bad_timedelta_str):
    with pytest.raises(TypeError):
        result = datefunc.parse_timedelta(bad_timedelta_str) 

def test_datefunc_timedelta_as_seconds():
    td = timedelta(days=1, hours=2, minutes=3, seconds=4, microseconds=5)
    result = datefunc.timedelta_as_seconds(td)
    assert result == 93784.000005

def test_datefunc_datetime_as_seconds():
    now = datetime.now()
    result = datefunc.datetime_as_seconds(now)
    assert result == now.timestamp()

def test_datefunc_datetime_as_seconds_utc():
    now = datetime.now(timezone.utc)
    result = datefunc.datetime_as_seconds(now)
    assert result == now.timestamp()

def test_datefunc_strptime():
    s = '2004-12-25 12:34:56'
    fmt = '%Y-%m-%d %H:%M:%S'
    result = datefunc.strptime(s, fmt) 
    assert result.strftime(fmt) == s




#!/usr/bin/env python2

from Transport 		import ProcessClient
from Transport 		import NewsPostMixin
from Transport.Util	import removeFile
from mx 			import DateTime

import sys
import os

class PostData(ProcessClient,NewsPostMixin):
	
	def __init__(self,argv):
		ProcessClient.__init__(self,argv)
		NewsPostMixin.__init__(self)

		self.pollrate	= self.getint('pollrate',60)
		self.filenames	= self.get('filenames','msg.txt').split()

	def run(self):
		
		counter=0

		while self.running:

			curdate	= DateTime.now()

			postnames = []

			for filename in self.filenames:

				filename = curdate.strftime(filename)
				name,ext = os.path.splitext(filename)

				message = 'Index: %d\n' % counter

				name,ext = os.path.splitext(filename)

				if ext=='.zip':
					open(name,'w').write(message)
					os.system('zip -o -qq %s %s' % (filename,name))
					os.remove(name)
				elif ext=='.gz':
					open(name,'w').write(message)
					os.system('gzip -f %s' % name)
				elif ext=='.bz2':
					open(name,'w').write(message)
					os.system('bzip2 -f %s' % name)
				else:
					open(filename,'w').write(message)

				postnames.append(filename)

			counter+=1

			self.log.info('Posting files: %s' % postnames)

			self.newsPoster.post(postnames)
			removeFile(postnames)

			self.wait(self.pollrate)

if __name__ == '__main__':
	PostData(sys.argv).run()
	



#!/usr/bin/env python3
"""Transport XMLRPC Service"""

##########################################################################
#
#   Transport XLMRPC Service
#
#   This is the base class for the different XML-RPC based services
#   used in the transport network. It interfaces with the directory
#   service to bootstrap the local server.
#
#   2005-11-09  Todd Valentic
#               Initial implementation.
#
#   2006-01-22  Todd Valentic
#               Added intospection registration
#
#   2006-02-25  Todd Valentic
#               Added idle function and timeout.
#
#   2006-06-14  Todd Valentic
#               Use label in directory server for ident.
#
#   2006-07-17  Todd Valentic
#               Added try..except around request handle to catch
#                   connection breaks.
#
#   2016-07-10  Todd Valentic
#               Python 2.6+ has a more sophisticated serve_forver()
#                   loop that ensures system calls complete and thus
#                   results in the old implementation blocking forever.
#                   Now we handle both the older and newer versions.
#
#   2016-10-31  Todd Valentic
#               Modify previous change to work on Python 2.4 as well.
#
#   2016-11-08  Todd Valentic
#               Remove call to undefined shutdown() method.
#               Fix type in assignment for server_forever for < python 2.6
#
#   2017-01-10	Todd Valentic
# 		Restore call to shutdown() method - needed to exit,
# 		    otherwise we deadlock. The function is inherited
# 		    from SimpleXMLRPCServer
#
#   2017-06-08  Todd Valentic
#               Rename ident() function to prevent conflict with
#                   threading.Thread.ident when this class is mixed in with
#                   Thread. This problem showed up in Python 2.6+
#
#   2019-08-06  Todd Valentic
#               Add timeout to RequestHandler
#               Set allow_none
#
#   2020-10-09  Todd Valentic
#               Python3: xmlrpc.server
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#
#   2023-07-04  Todd Valentic
#               Convert from mixin class
#
##########################################################################

import threading
import xmlrpc.client

from xmlrpc.server import SimpleXMLRPCServer
from xmlrpc.server import SimpleXMLRPCRequestHandler

from . import Directory

SimpleXMLRPCRequestHandler.timeout = 5


class XMLRPCServer(SimpleXMLRPCServer):
    """Data Transport XMLRPC Server"""

    # pylint: disable=too-many-arguments

    def __init__(
        self, parent, queue_size=20, port=None, label=None, callback=None, timeout=1
    ):
        self.parent = parent
        self.callback = callback
        self.timeout = timeout

        if port is None:
            # This is a normal service client

            self.directory = Directory(parent)

            servicename = parent.config.get("service.name")

            if not servicename:
                parent.abort("No service.name found")

            port = self.directory.get(servicename, "port")

            try:
                label = self.directory.get(servicename, "label")
            except xmlrpc.client.Fault:
                label = None

        self.allow_reuse_address = True
        self.request_queue_size = queue_size

        SimpleXMLRPCServer.__init__(
            self, ("", port), allow_none=True, logRequests=False
        )

        self.register_introspection_functions()
        self.register_function(self.my_ident, "ident")
        self.register_function(self.status)

        parent.log.info(f"Listening on port {port}")

        if label:
            self.servicelabel = label
        else:
            self.servicelabel = "No description available"

    def my_ident(self):
        """Service desciption"""
        return self.servicelabel

    def status(self):
        """Indicate active"""
        return 1

    def server_thread(self):
        """Run server in separate thread"""

        try:
            self.serve_forever()
        except:  # pylint: disable=bare-except
            pass
        finally:
            self.server_close()

    def main(self):
        """Run server"""

        self.parent.log.info("Starting operations")

        thread = threading.Thread(None, self.server_thread)
        thread.start()

        while self.parent.wait(self.timeout):
            if self.callback:
                self.callback()

        self.shutdown()
        thread.join()

        self.parent.log.info("Finished")



#!/usr/bin/env python3
"""Synchronized newsgroup poller"""

###########################################################################
#
#   Synchronized News Group Poller
#
#   This class provides a framework for polling multiple news groups
#   and synchronizing the message stream. A user provided callback
#   function is used to compare the next available message from each
#   polled news group. The function returns the preferred message. For
#   example, the message times might be compared so that the oldest
#   message is processed next. Using a synchronization function like
#   this is necessary when performing functions based on a stream of
#   multiple asynchonous events or when processing a backlog of
#   messages.
#
#   You can customize this class in a couple of ways. The most common
#   synchronization mechanism is based on the message times. The
#   default method compareMessage() checks these time stamps and
#   selects the news group with the oldest time stamp. You can
#   override the get_timestamp() method if the message times are not in
#   the standard headers. To do synchronization on another parameter,
#   you can override compare_messages(). It returns the poller object
#   corresponding to the news group you want to process. The messages
#   list holds the currently queued message for each news group.
#
#   2005-05-07  Todd Valentic
#               Initial implementation.
#
#   2006-02-03  Todd Valentic
#               Added valid_message default method.
#
#   2009-05-04  Todd Valentic
#               Added catchup and reset to _run_pollers() interface.
#                   These have been missing for awhile (NewsPollMixin
#                   changed the signature). The values are not used here.
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#               NewsPoller
#
#   2023-07-15  Todd Valentic
#               Updated for transport3 / python3
#               Converted to just be a specalized NewsPoller
#
###########################################################################

from datatransport import NewsPoller
from datatransport import newstool


class SyncPoller(NewsPoller):
    """Synchronized NewsPoller"""

    def __init__(self, *p, **kw):
        NewsPoller.__init__(self, *p, **kw)

        self.messages = {}

        for poller in self.news_pollers:
            self.messages[poller] = None

    def get_timestamp(self, message):
        """Get message timestamp"""

        return newstool.message_date(message)

    def valid_message(self, _message):
        """Check if message is valid"""

        return True

    def find_oldest_message(self):
        """Search for oldest message"""

        oldest_time = None
        oldest_poller = None

        for poller, message in self.messages.items():
            if not message:
                continue
            timestamp = self.get_timestamp(message)
            if oldest_time is None or timestamp < oldest_time:
                oldest_time = timestamp
                oldest_poller = poller

        return oldest_poller

    def get_next_message(self, poller):
        """Get the next *valid* message from the news group.
        Return None if no messages are available."""

        while self.is_running():
            message = poller.get_next_message()

            if message is None:
                break

            if self.valid_message(message):
                break

            # The message is not valid (based on some user
            # criteria). Mark the message as read and try again

            poller.mark_message_read(message)

        return message

    def refresh_messages(self):
        """Refresh message queue"""

        for poller in self.news_pollers:
            if self.messages[poller] is None:
                self.messages[poller] = self.get_next_message(poller)

    def run_pollers(self, catchup, reset):
        """Run each poller, synchronized"""

        while self.is_running():
            self.refresh_messages()

            if not self.is_running():
                break

            # Find the preferred message to process

            poller = self.find_oldest_message()

            if not self.is_running():
                break

            # Run until all messages are processed

            if poller is None:
                break

            poller.process_article(self.messages[poller])
            self.messages[poller] = None



#!/usr/bin/env python
"""Process Group"""

#############################################################################
#
#   ProcessGroup
#
#   2005-04-04  Todd Valentic
#               Added ability to load extra config files.
#
#   2005-06-22  Todd Valentic
#               Changed default PATH to be /usr/local/bin:/bin:/usr/bin
#
#   2005-11-30  Todd Valentic
#               Show configuration files in debug log output.
#
#   2006-02-03  Todd Valentic
#               Code cleanups. Removed deprecated verbose. Removed
#                   unused references to grouphome.
#
#   2007-03-17  Todd Valentic
#               Use os.path.sep and pick up config files in group root dir
#
#   2008-11-12  Todd Valentic
#               Set defaults for autostart and group label. If the
#                   group was unset, it defaulted to None which wasn't
#                   passed through the XML-RPC interface.
#
#               Pass parent log to record failures before our logging
#                   is setup.
#
#   2009-05-04  Todd Valentic
#               Use path.exec config option to set default path.
#
#   2009-05-11  Todd Valentic
#               Use builtin set type
#
#   2010-01-19  Todd Valentic
#               Added priority config parameters
#
#   2016-07-10  Todd Valentic
#               Set pid to 0 on logout
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#
#   2023-07-09  Todd Valentic
#               Use TransportConfig find_config_files()
#               Use place holde {client.name} when showing config in debug 
#
#   2023-07-20  Todd Valentic
#               Change error to debug if no clients listed
#               Reduce default shutdown_timeout to 30s
#               Change shutdown_timeout to timedelta 
#
#   2023-08-14  Todd Valentic
#               Make sure group.home in PYTHONPATH
#
#############################################################################

import configparser
import copy
import fnmatch
import glob
import logging
import os
import sys
import time

from . import TransportConfig
from .transportlogger import create_transport_logger


class ClientInfo:
    """Process client tracker"""

    def __init__(self, name, label, group):
        self.name = name
        self.label = label
        self.pid = 0
        self.group = group

    def alive(self):
        """Check if the process is still alive"""

        if self.pid and str(self.pid) in os.listdir("/proc"):
            return True

        self.pid = 0
        return False

    def stop(self):
        """Stop the process via TERM signal"""

        try:
            os.kill(self.pid, 15)
        except (PermissionError, ProcessLookupError) as err:
            self.group.log.info("PID %d: %s", self.pid, str(err))

    def kill(self):
        """Stop the process via KILL signal"""

        try:
            os.kill(self.pid, 9)
            self.pid = 0
        except (PermissionError, ProcessLookupError) as err:
            self.group.log.info("PID %d: %s", self.pid, str(err))


class ProcessGroup:
    """Manage groups of processes"""

    def __init__(self, name, queue, parent_log):
        self.name = name
        self.clients = {}
        self.running = False
        self.log = None
        self.queue = queue
        self.parent_log = parent_log

        self.environ = None
        self.config = None
        self.configfiles = None
        self.autostart = None
        self.grouplabel = None
        self.start_priority = None
        self.stop_priority = None
        self.shutdown_timeout = None
        self.report_rate = None

        self.reload()

    def reload(self):
        """Reload configuration files"""

        config_files = self.load_config()
        self.setup_log()
        self.setup_environ()
        self.load_clients()

        self.show_config(config_files)

        return True

    def show_config(self, config_files):
        """Display configuration options to log. Used for debugging"""

        self.log.debug("Configuration files:")
        for filename in config_files:
            self.log.debug("  - %s", filename)

        self.log.debug("Configuration values:")

        for section in self.config.sections():
            self.log.debug("  [%s]", section)

            for option in self.config.options(section):
                try:
                    value = self.config.get(section, option)
                    self.log.debug("    %s: %s", option, value)
                except configparser.Error as err:
                    self.log.debug("    %s: <-- ERROR (%s)", option, str(err))

    def setup_environ(self):
        """Setup environment variables"""

        self.log.debug("Environment:")

        self.environ = copy.deepcopy(os.environ)

        path = [os.path.dirname(sys.executable)]
        path.extend(self.config.get_list("ProcessGroup", "path.exec", sep=":"))
        path.extend(self.config.get_list("ProcessGroup", "group.bin", sep=":"))
        path.extend(self.config.get_list("ProcessGroup", "path.bin", sep=":"))

        # Filter out empty strings and duplicates

        filtered_path = []
        for entry in path:
            if entry and entry not in filtered_path:
                filtered_path.append(entry)

        self.environ["PATH"] = ":".join(filtered_path)

        options = self.config.options("ProcessGroup")

        setoptions = fnmatch.filter(options, "environ.set.*")
        addoptions = fnmatch.filter(options, "environ.add.*")

        for option in setoptions:
            value = self.config.get("ProcessGroup", option)
            var = option[12:].upper()
            self.environ[var] = value
            self.log.debug("  set: %s=%s", var, value)

        for option in addoptions:
            value = self.config.get("ProcessGroup", option)
            var = option[12:].upper()
            if var in self.environ:
                # only add if not there
                if self.environ[var].find(value) == -1:
                    self.environ[var] += ":" + value
            else:
                self.environ[var] = value
            self.log.debug("  add: %s=%s", var, self.environ[var])

        if 'PYTHONPATH' in os.environ:
            pythonpath = os.environ['PYTHONPATH'].split(':')
        else:
            pythonpath = []

        grouphome = self.config.get("ProcessGroup", "group.home")

        if grouphome not in pythonpath:
            pythonpath.append(grouphome)
            os.environ['PYTHONPATH'] = ':'.join(pythonpath)

        for key, value in self.environ.items():
            self.log.debug("  %s = %s", key, value)

    def setup_log(self):
        """Setup logging"""

        if not self.log:
            self.log = create_transport_logger(self.config["ProcessGroup"], self.name)
            self.log.info("---------- START ----------")

        level = self.config["ProcessGroup"].get("log.level", "info")
        self.log.info("Setting log level to %s", level)

        if level == "warning":
            self.log.setLevel(logging.WARNING)
        elif level == "info":
            self.log.setLevel(logging.INFO)
        else:
            self.log.setLevel(logging.DEBUG)

    def load_config(self):
        """Load configuration"""

        defaults = {
            "client.name": "{client.name}",
            "group.name": self.name,
            "group.basename": os.path.basename(self.name),
            "group.dirname": os.path.dirname(self.name),
        }

        self.config = TransportConfig(defaults)
        basepath = self.config["TransportServer"].get_path("path.groups")
        hostname = self.config["TransportServer"].get("hostname")
        config_files = self.config.find_config_files(self.name, basepath, hostname) 

        for filename in config_files:
            self.config.read(filename)

        processgroup = self.config["ProcessGroup"]

        self.autostart = processgroup.get_boolean("autostart", False)
        self.grouplabel = processgroup.get("label", self.name)
        self.start_priority = processgroup.get_int("priority.start", 50)
        self.stop_priority = processgroup.get_int("priority.stop", 50)
        self.shutdown_timeout = processgroup.get_timedelta("shutdown.timeout", 30)
        self.report_rate = processgroup.get_int("shutdown.report.rate", 15)

        return config_files

    def load_clients(self):
        """Load clients"""

        clientlist = self.config.get_list("ProcessGroup", "clients")

        staleclients = set(self.clients).difference(clientlist)
        newclients = set(clientlist).difference(self.clients)
        curclients = set(clientlist).intersection(self.clients)

        self.log.debug("Client list: %s", clientlist)
        self.log.debug("  remove   : %s", staleclients)
        self.log.debug("  add      : %s", newclients)
        self.log.debug("  existing : %s", curclients)

        for name in staleclients:
            self.remove_client(name)

        for name in newclients:
            self.add_client(name)

        for name in curclients:
            # Only thing we update is the label
            self.clients[name].label = self.config[name].get("label", "")

    def check_client(self, name, action):
        """Check if client has been definied"""

        if name not in self.clients:
            self.log.error("Request to %s an unknown client: %s", action, name)
            raise NameError(name)

    def login(self, name, pid):
        """Login a client"""

        self.check_client(name, "login")
        self.log.info("Logging in client: %s (pid=%d)", name, pid)
        self.clients[name].pid = pid

    def logout(self, name, pid):
        """Logout a client"""

        self.check_client(name, "logout")
        self.log.info("Logging out client: %s (pid=%d)", name, pid)
        self.clients[name].pid = 0

    def find_command(self, client):
        """Check if clients command file exists"""

        name = self.config[client].get("command")

        if not name:
            self.log.error("No command defined for client: %s", client)
            return None

        paths = self.environ["PATH"].split(":")

        for path in paths:
            cmd = os.path.join(path, name)
            if os.path.isfile(cmd):
                return cmd

        self.log.error("Cannot find program in search path!")
        self.log.error("  client: %s", client)
        self.log.error("  command: %s", name)
        self.log.error("  path: %s", self.environ["PATH"])

        return None

    def start_clients(self, clientnames=None, userargs=""):
        """Start running a process group's clients"""

        self.reload()

        if not clientnames:
            try:
                clientnames = self.config["ProcessGroup"].get("clients.start")
                clientnames = clientnames.split()
            except (configparser.Error, AttributeError):
                clientnames = self.clients.keys()

        if not clientnames:
            self.log.debug("No clients listed")
            return

        clientnames = set(clientnames).intersection(self.clients)

        for clientname in clientnames:
            cmd = self.find_command(clientname)
            args = [cmd, self.name, clientname]
            args.extend(userargs.split())

            if cmd:
                self.log.info("Starting client %s", clientname)
                self.log.debug("  %s", " ".join(args))
                self.queue.put((cmd, args, self.environ))

    def stop_clients(self, names):
        """Stop the specificed clients"""

        names = set(names).intersection(self.clients)
        num_running = 0

        for name in names:
            if self.clients[name].alive():
                self.parent_log.info("Stopping client %s", name)
                self.clients[name].stop()
                num_running += 1

        timeout = time.time() + self.shutdown_timeout.total_seconds()
        report_time = time.time() + self.report_rate

        while time.time() < timeout and num_running > 0:
            time.sleep(1)
            num_running = 0
            for name in names:
                if self.clients[name].alive():
                    if time.time() > report_time:
                        pid = self.clients[name].pid
                        self.parent_log.info("  - still running: %s (%d)", name, pid)
                        report_time = time.time() + self.report_rate
                    num_running += 1

        if num_running:
            for name in names:
                if self.clients[name].alive():
                    self.parent_log.info("Killing unresponsive client %s", name)
                    self.clients[name].kill()

        return True

    def start_client(self, name, args=""):
        """Start a process client"""

        self.check_client(name, "start")
        self.start_clients([name], args)

    def stop_client(self, name):
        """Stop a process client"""

        self.check_client(name, "stop")
        return self.stop_clients([name])

    def add_client(self, name):
        """Add a new client"""

        label = self.config[name].get("label", "")
        self.clients[name] = ClientInfo(name, label, self.name)
        self.log.info("Adding client %s", name)
        return True

    def remove_client(self, name):
        """Remove a client"""

        self.check_client(name, "remove")

        if self.clients[name].alive():
            self.clients[name].stop()

        del self.clients[name]
        self.log.info("Removing client %s", name)

        return True

    def start(self):
        """Start listed clients"""
        self.start_clients()

    def stop(self):
        """Stop running clients"""
        return self.stop_clients(list(self.clients))

    def list_clients(self):
        """List available clients"""

        results = {}

        for client in self.clients.values():
            results[client.name] = (client.label, client.pid)

        return results



#!/usr/bin/env python

############################################################################
#
#   Counter 
#
#   Write something to the log file 
#
#   History:
#   
#   1.0.0   2004-02-08  Todd Valentic   
#           Initial implementation
#
############################################################################

from    Transport   import ProcessClient

import  sys

class Counter(ProcessClient):

    def __init__(self,argv):
        ProcessClient.__init__(self,argv)

    def run(self):

        rate    = self.getDeltaTime('rate',1)
        offset  = self.getDeltaTime('offset',0)
        counter = 0

        while self.wait(rate,offset,sync=True): 
            self.log.info('Counter = %d' % counter)
            counter += 1

if __name__ == '__main__':
    Counter(sys.argv).run()




#!/usr/bin/env python
"""Service client example"""

##########################################################################
#
#   Example client calling a service
#
#   2022-11-02  Todd Valentic
#               Initial implementation
#
##########################################################################

import sys

from datatransport import ProcessClient
from datatransport import Directory


class EchoClient(ProcessClient):
    """Client calling the echo service"""

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        self.directory = Directory(self)

        self.rate = self.config.get_rate("rate", "10s")

        self.log.info('echo client')

        self.echo = self.directory.connect("echo")

    def run(self):
        self.log.info("Starting")
        try:
            self._run()
        except BaseException as err:
            self.log.exception(str(err))
        self.log.info("Stopping")

    def _run(self):
        """Main loop"""

        while self.wait(self.rate):
            result = self.echo.emit(f"Helloooooo from {self.name}")
            self.log.info("Call returned '%s'", result)


if __name__ == "__main__":
    EchoClient(sys.argv).run()



#!/usr/bin/env python3
"""Example SyncPoller"""

import sys

from datatransport import ProcessClient
from datatransport.apps import SyncPoller 

class Poller(ProcessClient):
    """SyncPoller Process Client"""
    
    def __init__(self, args):
        ProcessClient.__init__(self, args)

        self.poller = SyncPoller(self, callback=self.process)
        self.main = self.poller.main

    def process(self, message): 
        """Process handler"""

        body = message.get_payload() 
        self.log.info(body)

if __name__ == '__main__':
    Poller(sys.argv).run()




#!/usr/bin/env python3
"""Disk Monitor Component"""

##########################################################################
#
#   DiskMonitor
#
#   This process client monitors the system disk usage reported in the
#   output messages from the ResourceMonitor client running in the
#   ServerMonitor process group. A trouble message will be sent when
#   disk space is running low.
#
#   2004-02-01	Todd Valentic
#               Initial implementation.
#
#   2008-11-13  Todd Valentic
#               Use SafeConfigParser
#
#   2016-12-23  Todd Valentic
#               Use external texttable
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#               Python3 updates
#                   ConfigParser -> sapphire.Parser
#                   StringIO -> io.StringIO
#                   sizeDesc -> size_desc
#                   NewsPoster, NewsPoller
#
#   2023-07-27  Todd Valentic
#               Updated for transport3 / python3
#
##########################################################################

import fnmatch
import pathlib
import sys

import texttable

from datatransport import ProcessClient
from datatransport import NewsPoster
from datatransport import NewsPoller
from datatransport import newstool
from datatransport.utilities import size_desc

import sapphire_config as sapphire


class DiskMonitor(ProcessClient):
    """Process Client"""

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        self.news_poster = NewsPoster(self)
        self.news_poller = NewsPoller(self, callback=self.process)
        self.main = self.news_poller.main

        self.cachefile = pathlib.Path("history")
        self.load_history()

        sysmnts = "/dev* /run* /sys*"

        self.includes = self.config.get_list("mounts.include", "*")
        self.excludes = self.config.get_list("mounts.exclude", sysmnts)
        self.max_used_pct = self.config.get_float("max.percent.used", 95)

    def load_history(self):
        """Load history from cache"""

        self.history = sapphire.Parser()
        self.history.read(self.cachefile)

    def save_history(self):
        """Save history to cache"""

        with self.cachefile.open("w", encoding="utf-8") as f:
            self.history.write(f)

    def update_history(self, prefix, stats, mount):
        """Update history"""

        if not self.history.has_section(mount):
            self.history.add_section(mount)

        for option in stats.options(mount):
            value = stats.get(mount, option)
            self.history.set(mount, f"{prefix}.{option}", value)

    def checkmount(self, stats, mount):
        """Check mounted filesystem"""

        if self.history.has_section(mount):
            prevalarm = self.history.get_boolean(mount, "last.alarm")
        else:
            prevalarm = False

        usedpct = stats.get_float(mount, "usedpct")
        curalarm = usedpct >= self.max_used_pct

        setalarm = not prevalarm and curalarm
        clearalarm = prevalarm and not curalarm

        if setalarm:
            self.log.info("Warning: %s is %d%% full", mount, usedpct)
            notes = "Alarm set"
        elif clearalarm:
            self.log.info("Cleared: %s is now only %d%% full", mount, usedpct)
            notes = "Cleared alarm"
        elif curalarm and prevalarm:
            notes = "Low space alarm"
        else:
            notes = ""

        self.update_history("last", stats, mount)
        self.history.set(mount, "last.alarm", str(curalarm))
        self.history.set(mount, "last.notes", notes)

        return setalarm or clearalarm

    def check(self, stats):
        """Check mounted filesystems"""

        self.log.debug("Checking disk usage (limit is %d%%)", self.max_used_pct)

        # Patchup for development debugging
        if not stats.has_section("System"):
            stats.add_section("System")

        mounts = stats.get_list("System", "mounts")

        for include in self.includes:
            mounts = [m for m in mounts if fnmatch.fnmatch(m, include)]
        for exclude in self.excludes:
            mounts = [m for m in mounts if not fnmatch.fnmatch(m, exclude)]

        report = 0

        for mount in mounts:
            report += self.checkmount(stats, mount)

        if report:
            message = self.alert_message(mounts)
            try:
                self.news_poster.post_text(message, date=self.timestamp)
            except Exception as e: # pylint: disable=broad-exception-caught
                self.log.exception("Problem posting message: %s", e)

    def alert_message(self, mounts):
        """Return an alert message"""

        timestr = self.timestamp.strftime("%Y-%m-%d %H:%M")

        message = "\n"
        message += f"Disk Space Report - {timestr}\n\n"

        table = texttable.Texttable()
        table.header(["Mount", "Total", "Used", "Avail", "% Full", "Notes"])
        table.set_cols_align(["l", "r", "r", "r", "r", "l"])

        for mount in mounts:
            total = self.history.get_float(mount, "last.totalbytes")
            used = self.history.get_float(mount, "last.usedbytes")
            free = self.history.get_float(mount, "last.freebytes")
            usedpct = self.history.get_float(mount, "last.usedpct")

            row = [mount]
            row.append(size_desc(total))
            row.append(size_desc(used))
            row.append(size_desc(free))
            row.append(f"{usedpct:0.1f}%")
            row.append(self.history.get(mount, "last.notes"))

            table.add_row(row)

        message += table.draw()
        message += "\n"

        return message

    def process(self, message):
        """Process handler"""

        stats = newstool.as_config(message)

        self.timestamp = newstool.message_date(message)
        self.history.set("DEFAULT", "last.time", str(self.timestamp))

        self.check(stats)
        self.save_history()


def main():
    """Script entry point"""
    DiskMonitor(sys.argv).run()



#!/usr/bin/env python3
"""Group Control Component"""

###########################################################################
#
#  GroupControl
#
#  This script polls a "control" newsgroup looking for messages to start
#  and stop process groups.
#
#  History:
#
#   1.0.0   2000-01-19  TAV
#           Initial implementation
#
#   1.0.1   2000-02-22  TAV
#           Added the maillist reporting feature.
#
#   1.0.2   2000-02-26  TAV
#           Improved wording and look of the mailing list message.
#
#   1.0.3   2000-04-25  TAV
#           Added connect() method to retry connecting to server incase it
#               is busy during the initial contact.
#
#   1.0.4   2000-04-29  TAV
#           Fixed bug in connect call.
#
#   1.0.5   2001-11-13  TAV
#           Added prefix and postfix config options.
#
#   1.0.6   2001-12-26  TAV
#           Changed tav -> sri
#
#   1.0.7   2002-01-25  Todd Valentic
#           Updated to new NewsPollMixin interface.
#
#   1.0.8   2002-08-27  Todd Valentic
#           Setup through configure script.
#           sri.transport -> Transport
#           use socket.getfqdn() versus sri.util.hostname()
#
#   1.0.9   2003-03-21  Todd Valentic
#           Code cleanups. Use self.wait in connect()
#           Use string methods instead of string module
#           Split out baseclass from driver
#
#   1.0.10  2003-05-09  Todd Valentic
#           Moved the connection code up into __init__()
#               we were sleeping 60 seconds on the first
#               message process otherwise.
#
#   1.0.11  2003-06-30  Todd Valentic
#           Updated to new NewsPoller interface.
#
#   1.0.12  2004-08-08  Todd Valentic
#           Use new XML-RPC interface to transport server.
#
#   1.0.13  2004-12-27  Todd Valentic
#           Convert from mx.DateTime to datetime
#
#   1.0.14  2006-01-18  Todd Valentic
#           Minor code cleanups
#
#   1.0.15  2006-10-26  Todd Valentic
#           Use new CurrentTime method
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#               Python3 migration:
#                   exception handling
#                   NewsPoller, NewsPoster
#
#   2023-07-28  Todd Valentic
#               Updated for transport3 / python3
#
###########################################################################

import nntplib
import socket
import sys
import xmlrpc.client

from pathlib import Path

from datatransport import ProcessClient
from datatransport import NewsPoller
from datatransport import NewsPoster
from datatransport import newstool


class GroupControl(ProcessClient):
    """Process Client"""

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        self.news_poster = NewsPoster(self)
        self.news_poller = NewsPoller(self, callback=self.process)
        self.main = self.news_poller.main

        self.prefix = self.config.get_path("prefix", "")
        self.postfix = self.config.get_path("postfix", "")

        self.wait_for_server()

    def wait_for_server(self):
        """Connect to transport server"""

        self.log.info("Connecting to transport server")

        while self.is_running():
            try:
                self.server.status()
                return
            except xmlrpc.client.Fault:
                self.log.error("  - waiting 30 seconds to try again.")

            if not self.wait(30):
                break

    def post_message(self, action, group):
        """Post notification message"""

        hostname = socket.getfqdn()

        body = []
        body.append("")
        body.append("  The following action has been performed:")
        body.append("")
        body.append(f"      Command  : {action}")
        body.append(f"      Group    : {group}")
        body.append(f"      Time     : {self.now()}")
        body.append(f"      Server   : {hostname}")
        body.append("")

        body = "\n".join(body)

        header = f"{action} {group}"

        try:
            self.news_poster.set_subject(header)
            self.news_poster.post_text(body)
        except nntplib.NNTPError as e:
            self.log.error("Error posting message: %s", e)

    def start_group(self, name):
        """Start process group"""

        self.log.info("Start request for %s", name)

        try:
            self.server.startgroup(name)
        except xmlrpc.client.Fault as err:
            self.log.error("Failed to start: %s", err)
            return

        self.post_message("start", name)

    def stop_group(self, name):
        """Stop a process group"""

        self.log.info("Stop request for %s", name)

        try:
            self.server.stopgroup(name)
        except xmlrpc.client.Fault as err:
            self.log.info("Failed to stop: %s", err)
            return

        self.post_message("stop", name)

    def process(self, message):
        """Process message handler"""

        filenames = newstool.save_files(message)

        for filename in filenames:
            if self.is_stopped():
                return

            contents = filename.read_text(encoding="utf-8")

            for line in contents.split("\n"):
                try:
                    command, name = line.split("=")
                except ValueError:
                    continue

                command = command.strip().lower()
                name = name.strip()

                groupname = str(Path(self.prefix, name, self.postfix))

                if command == "start":
                    self.start_group(groupname)
                elif command == "stop":
                    self.stop_group(groupname)
                else:
                    self.log.error("Unknown request: %s", command)

            filename.unlink()


def main():
    """Script entry point"""
    GroupControl(sys.argv).run()



#!/usr/bin/env python

#####################################################################
#
#   Demonstation of the eventmonitor class
#
#   Track the up/down state of servers.
#
#   2009-04-30  Todd Valentic
#               Initial implementation.
#
#####################################################################

from Transport              import ProcessClient
from Transport              import ConfigComponent
from Transport.Components   import EventMonitor

import sys
import os
import commands
import StringIO
import pytz
import datetime
import random

#####################################################################
#
#   Each thing that we track inherits from EventMonitor.Member.
#   It is a standard ConfigComponent. You need to define the
#   different states that are tracked in the "states" option.
#   The initial state is in states.unknown (and defaults to
#   "unknown"). It is not necessary to list the unknown state
#   in the states list, it will added automatically if it isn't
#   there.
#
#   The full list of options is:
#
#       .states = space separated list of names
#       .states.unknown = name of undefined state
#       .maxhistory = number of previous transitions to keep
#
#   The class needs to respond to save() and load() calls which
#   saves the state into a DefaultConfigParser object. You can
#   track class-specific information and save into the config
#   object as well.
#
#####################################################################

class Host(EventMonitor.Member):

    def __init__(self,*pos,**kw):
        EventMonitor.Member.__init__(self,'host',*pos,**kw)

        self.hostname   = self.get('hostname',self.name)
        self.label      = self.get('label',self.name)
        self.loadlevel  = 0

    def save(self,config):
        EventMonitor.Member.save(self,config)

        config.set(self.name,'host.name',self.hostname)
        config.set(self.name,'host.label',self.label)

        config.set(self.name,'loadlevel',str(self.loadlevel))

    def load(self,config):
        EventMonitor.Member.load(self,config)

        self.levelload = config.getfloat(self.name,'loadlevel',0.0)

    def check(self,up):

        if up:
            self.loadlevel = random.randrange(10)
            self.update('online')
        else:
            self.update('offline')
            self.loadlevel = 0

#####################################################################
#
#   Main application class
#
#   Include an EventMonitor() oject for tracking. Pass it a
#   dictionary of EventMontiorMembers, which can be obtained
#   from getComponentsDict().
#
#####################################################################

class HostMonitor (ProcessClient):

    def __init__(self,argv):
        ProcessClient.__init__(self,argv)

        self.hosts      = self.getComponentsDict('hosts',Host)
        self.tracker    = EventMonitor.EventMonitor(self.hosts)

        self.log.info('Target hosts:')
        for name,host in self.hosts.items():
            self.log.info('  - %s (%s)' % (name,host.hostname))

        # For testing, we want to start with a clean slate.
        # Normally, you would load the tracker state here.

        # self.loadTracker()

    def loadTracker(self):
        self.tracker.load('tracker')

    def saveTracker(self):
        self.tracker.save('tracker')

        # Post results to news groups here...

    def run(self):

        self.log.info('Setting server1 online, server2 offline')
        self.hosts['server1'].update('online')
        self.hosts['server2'].update('offline')

        self.log.info('Tracker changes: %s' % self.tracker.changed())

        self.wait(2)

        self.log.info('Setting server3 offline, server3 online')
        self.hosts['server1'].update('online')
        self.hosts['server2'].update('offline')
        self.hosts['server3'].update('offline')
        self.hosts['server3'].update('online')

        self.log.info('Tracker changes: %s' % self.tracker.changed())

        self.log.info('Current State:')

        for host in self.hosts.values():
            curState = host.curState
            self.log.info('  %s' % host.name)
            self.log.info('  %s' % curState)
            self.log.info('    start: %s' % host.states[curState].starttime)
            self.log.info('    elapsed: %s' % host.states[curState].elapsed)
            self.log.info('  history:')

            for entry in host.history:
                self.log.info('    %s' % entry)

        self.saveTracker()

        self.exit('Finished')

if __name__ == '__main__':
    HostMonitor(sys.argv).run()



#!/usr/bin/env python
"""Test setting environment"""

############################################################################
#
#   Environ
#
#   This script displays the environment variables is sees at startup.
#
#   History:
#
#   2002-12-19  Todd Valentic
#               Initial implementation
#
#   2022-11-02  Todd Valentic
#               Python3 update
#
############################################################################

import os
import sys

from datatransport import ProcessClient


class DisplayEnviron(ProcessClient):
    """Display environment"""

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

    def init(self):
        super().init()

        self.print_environ()

    def print_environ(self):
        """Print environment to log"""

        self.log.info("Current Environment:")

        for key in sorted(os.environ):
            self.log.info("%s = %s", key, os.environ[key])

    def main(self):
        """Main loop"""

        while self.wait(60):
            pass


if __name__ == "__main__":
    DisplayEnviron(sys.argv).run()



#!/usr/bin/env python
"""Console Command"""

###########################################################################
#
#   XML-RPC Service Command Console
#
#   Run commands from an XML-RPC service.
#
#   2006-02-20  Todd Valentic
#               Initial implementation.
#
#   2007-03-11  Todd Valentic
#               Added try..except around connect in do_run()
#
#   2022-10-25  Todd Valentic
#               Python3 port
#               optparse -> argparse
#
###########################################################################

import argparse
import cmd
import os
import sys
import fnmatch
import xmlrpc.client
import traceback
import socket

socket.setdefaulttimeout(100)


class Console(cmd.Cmd):
    def __init__(self, args):
        cmd.Cmd.__init__(self)

        self.intro = self.make_intro()
        self.prompt = "[not connected] >>> "
        self.options = args

        self.configdir = os.path.expanduser("~/.acorn")
        self.configfile = os.path.join(self.configdir, "history")

        if not os.path.exists(self.configdir):
            os.makedirs(self.configdir)

        self.loadHistory()

        self.serviceCache = {} 

    def connect(self, service):
        url = self.directory.get(service, "url")
        return xmlrpc.client.ServerProxy(url)

    def make_intro(self):

        width = 75
        intro = []
        intro.append("")
        intro.append("-" * width)
        intro.append("XML-RPC Service Control Console 1.0".center(width))
        intro.append('Type "help" for a list commands'.center(width))
        intro.append("-" * width)
        intro.append("")

        return "\n".join(intro)

    # -- History --------------------------------------------------------

    def loadHistory(self):

        self.historyCache = {}
        self.historyIndex = 0

        if not os.path.exists(self.configfile):
            return

        for entry in open(self.configfile):
            index, cmd = entry.split(":", 1)
            index = int(index)
            self.historyCache[index] = cmd.strip()
            self.historyIndex = max(self.historyIndex, index)

        self.trimHistory()

    def saveHistory(self):

        with open(self.configfile, "wt") as output:

            keys = sorted(self.historyCache)

            for index in keys:
                print(f"{index}: {self.historyCache[index]}", file=output)


    def trimHistory(self):

        cutoff = self.historyIndex - self.options.maxhistory + 1

        for index in list(self.historyCache.keys()):
            if index < cutoff:
                del self.historyCache[index]

    def do_clear(self, arg):
        """clear history cache"""

        self.historyCache = {}
        self.historyIndex = 0
        self.saveHistory()

    def do_shell(self, args):
        """Run previous commands"""

        args = [int(x) for x in args.split()]
        for index in args:
            if self.onecmd(self.historyCache[index]):
                return True

        return False

    def do_history(self, args):
        """history - print a list of commands that have been entered"""

        for index in sorted(self.historyCache):
            print(f"{index:2}: {self.historyCache[index]}")

        return False

    # -- Connection ------------------------------------------------------

    def do_connect(self, arg):
        """connect [directory [port]] -- connect to directory service"""

        arg = arg.split()

        if not arg:
            host, port = self.options.host, self.options.port
        elif len(arg) == 1:
            host, port = arg[0], self.options.port
        else:
            host, port = arg

        url = f"http://{host}:{port}"
        self.directory = xmlrpc.client.ServerProxy(url)

        try:
            self.directory.ident()
            print(f"Connected to directory at {host}:{port}")
        except:
            print(f"Error connecting to directory on {host}:{port}")

        self.prompt = f"[{host}:{port}] >>> "

        return self.onecmd("refresh")

    def do_refresh(self, arg):
        """refresh -- Refresh list of services for autocompletion"""

        print("Refreshing service list ...")
        self.serviceCache = {}

        try:
            for service in self.directory.list():
                label = self.directory.get(service, "label")
                host = self.directory.get(service, "host")
                port = self.directory.get(service, "port")
                self.serviceCache[service] = (label, host, port)
        except:
            print("Error connecting to server")

        print(f"Refreshed -- {len(self.serviceCache)} services")

        return False

    # -- Listing ------------------------------------------------------

    def do_services(self, arg):
        """services -- list registered services"""

        # If we do not have a cache, refresh first

        if not self.serviceCache:
            self.onecmd("refresh")

        if not self.serviceCache:
            print("No services are registered")
            return False

        names = sorted(self.serviceCache)

        print(f"There are {len(names)} services registered:")

        width = max([len(x) for x in names])

        for name in names:
            label, host, port = self.serviceCache[name]
            print(f"[{host}:{port}] {name.ljust(width)} - {label}")

        return False

    def do_list(self, arg):
        """list <service> -- list commands in a service"""

        if not arg:
            return self.onecmd("services")

        if arg not in self.serviceCache:
            print("no service by that name. need to refresh?")
            return False

        try:
            service = self.connect(arg)
            commands = sorted(service.system.listMethods())
        except:
            print("problem communicating with service")
            return False

        print(f"There are {len(commands)} commands in {arg}:")

        width = max([len(x) for x in commands])

        for name in commands:
            print(f"  {name}")

        return False

    def complete_list(self, text, line, begidx, endidx):
        names = self.serviceCache.keys()
        prefix = line.split()[1].strip()
        matches = fnmatch.filter(names, prefix + "*")
        matches = [name.replace(prefix, text) for name in matches]
        return matches

    # -- Execute ---------------------------------------------------------

    def do_run(self, arg):

        arg = arg.split()

        if len(arg) < 2:
            print("Need to specify service and method names")
            return False

        try:
            service = self.connect(arg[0])
        except:
            print(f"Failed to connect to {arg[0]}")
            return False

        method = arg[1]
        params = arg[2:]

        func = getattr(service, method)

        try:
            print(func(*params))
        except:
            print(f"Problem running command:")
            traceback.print_exc(1)

        return False

    # -- Control ---------------------------------------------------------

    def do_quit(self, arg):
        """quit -- terminates the application"""
        return True

    def preloop(self):
        cmd.Cmd.preloop(self)
        self.serviceCache = {}

    def postloop(self):
        cmd.Cmd.postloop(self)
        self.saveHistory()
        print("Exiting...")

    def precmd(self, line):
        if line and not line.startswith("!"):
            self.historyIndex += 1
            self.historyCache[self.historyIndex] = line.strip()
            self.trimHistory()
        return line

    def emptyline(self):
        pass

    # shortcuts

    do_EOF = do_quit
    do_q = do_quit
    do_exit = do_quit
    do_h = do_history

    complete_run = complete_list


def main():

    parser = argparse.ArgumentParser(
        description="Data Transport XMLRPC Service Console"
    )

    parser.add_argument(
        "-p",
        "--port",
        default=8411,
        type=int,
        help="port (default %(default)s)",
    )

    parser.add_argument(
        "-r",
        "--host",
        default="localhost",
        help="direcotry host (default %(default)s)",
    )

    parser.add_argument(
        "-d",
        "--maxhistory",
        default=100,
        type=int,
        metavar="COUNT",
        help="maximum history depth (default %(default)s",
    )

    args = parser.parse_args()

    Console(args).cmdloop()



#!/usr/bin/env python3
"""FileWatch component"""

# pylint: disable=broad-exception-caught

##########################################################################
#
#   FileWatch
#
#   This transport component script watches for new files to appear in
#   a specified directory and posts them into a given newsgroup. If the
#   newsgroup does not exist when the script starts, it will try to
#   create it. Multiple files can be listed and they will be posted as
#   a group of attachments in a single message once they are all present.
#   This is a useful feature if you have data files that belong together
#   and should stay bundled in the transport network. Once the files are
#   posted, they are removed by this script. An option exists to leave
#   the files as well. For the files to be removed, they need to have
#   write permission for the transport user.
#
#   Config options:
#
#       (includes ProcessClient and NewsPoster options)
#
#       pollrate    - Integer, rate at which to check for new files (seconds)
#       remove_files - [0,1] flag indicating if files should be removed
#       watchpath   - String, path to look for files.
#       watchfiles  - String, list of file names to look for. If multiple
#                     names are listed, they should be separated by spaces.
#                     The name can include normal shell wildcards.
#
#   1.0.0   2000-??-??  TAV
#           Initial implementation.
#
#   1.0.1   2001-09-07  TAV
#           Changed script to abort at start up if no files are listed.
#           Check to see if the file still exists before trying to remove it.
#             This happens when setting things up and the group doesn't
#             have permission to remove the file. You'll manually remove
#             it and fix up the script writing the file. In the mean time
#             filewatch would try to remove the nonexistent file and crash.
#           Changed setStatus() -> setStatusCode().
#
#   1.0.2   2002-01-25  Todd Valentic
#           Added check for self.running.
#
#   1.0.3   2002-08-27  Todd Valentic
#           Configured through configure script
#           sri.transport -> Transport
#
#
#   1.0.4   2002-12-12  Todd Valentic
#           Removed setting of state in network (this is old code and not used).
#           Catch errors in posting.
#           Handled error on removing file better.
#
#   1.0.5   2002-12-18  Todd Valentic
#           Changed option names to be more consistent:
#               watchpath   -> watch.path
#               watchfiles  -> watch.files
#               pollrate    -> watch.rate
#               remove_files -> watch.removefiles
#
#   1.0.6   2004-07-02  Todd Valentic
#           Change rate to be DeltaDateTime.
#           Fixed reporting error when files cannot be removed.
#
#   1.0.7   2005-02-08  Todd Valentic
#           Added grouping option.
#           Make sure to sort file list.
#           Code clean up.
#
#   1.0.8   2005-10-07  Todd Valentic
#           Make sure watch.files has a default empty config value.
#
#   2022-10-07  Todd Valentic
#               Python3 port:
#                   Reorder imports
#                   removeFile -> remove_file
#                   NewsPoster
#
#   2023-07026  Todd Valentic
#               Use new config interface
#
##########################################################################

import sys

from datatransport import ProcessClient
from datatransport import NewsPoster
from datatransport.utilities import remove_file


class FileWatch(ProcessClient):
    """Process Client"""

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        self.news_poster = NewsPoster(self)

        self.pollrate = self.config.get_rate("watch.rate", 60)
        self.remove_files = self.config.get_boolean("watch.removefiles", True)
        self.group_files = self.config.get_boolean("watch.groupfiles", False)

        self.watchpath = self.config.get_path("watch.path", ".")
        self.filespecs = self.config.get_list("watch.files")

        if not self.filespecs:
            self.abort("No watch files listed in the config file.")

        self.log.info("Posting to %s", self.config.get("post.newsgroup"))
        self.log.info("Watching path: %s", self.watchpath)
        self.log.info("Watching for: %s", self.filespecs)

    def find_files(self):
        """Find files to post"""

        readyfiles = []

        for filespec in self.filespecs:
            files = [f for f in self.watchpath.glob(filespec) if f.is_file()]
            readyfiles.extend(files)

        return sorted(readyfiles)

    def process(self):
        """Process files ready to post"""

        files = self.find_files()

        if not files:
            self.log.debug("No files present, sleeping")
            return

        if self.group_files:
            files = [files]

        for filegroup in files:
            try:
                starttime = self.now()
                self.news_poster.post(filegroup)
                elapsed = self.now() - starttime
                self.log.info("Files posted (%s): %s", elapsed, filegroup)
            except Exception:
                self.log.exception("Problem posting files")
                return

            if self.remove_files:
                try:
                    remove_file(filegroup)
                except Exception:
                    self.log.exception("Problem deleting file: %s", filegroup)

    def main(self):
        """Main application"""

        while self.wait(self.pollrate):
            self.process()


def main():
    """Script entry"""
    FileWatch(sys.argv).run()



#!/usr/bin/env python

from Transport.Components   import SyncPoller 

import sys

class Poller (SyncPoller):
    
    def __init__(self,argv):
        SyncPoller.__init__(self,argv,callback=self.process)

    def process(self,message): 
        body = message.get_payload() 
        self.log.info(body)

if __name__ == '__main__':
    Poller(sys.argv).run()




#!/usr/bin/env python
"""Post test data files"""

##########################################################################
#
#   Post test data files
#
#   2022-11-04  Todd Valentic
#               Initial implementation
#
##########################################################################

import bz2
import gzip
import pathlib
import sys
import zipfile

from datatransport import ProcessClient
from datatransport import NewsPoster
from datatransport.utilities import remove_file, make_path


class PostData(ProcessClient):
    """Data Poster"""

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

    def init(self):
        super().init()

        self.news_poster = NewsPoster(self)
        self.filenames = self.config.get_list("filenames")

    def save_zip(self, filename, message):
        """Save as zip file"""

        with zipfile.ZipFile(filename, "w") as archive:
            archive.writestr("content", message)

    def save_gzip(self, filename, message):
        """Save as gzip file"""

        with gzip.open(filename, "wt") as archive:
            archive.write(message)

    def save_bzip2(self, filename, message):
        """Save as bzip2 file"""

        with bz2.open(filename, "wt") as archive:
            archive.write(message)


    def main(self):
        """Main loop"""

        rate = self.config.get_rate("rate", 60)

        counter = 0

        while self.wait(rate):

            now = self.now()
            message = f"Index: {counter}"

            postnames = []

            for name in self.filenames:

                filename = pathlib.Path(now.strftime(name))
                make_path(filename)

                if filename.suffix == ".zip":
                    self.save_zip(filename, message)

                elif filename.suffix == ".gz":
                    self.save_gzip(filename, message)

                elif filename.suffix == ".bz2":
                    self.save_bzip2(filename, message)

                else:
                    filename.write_text(message, encoding='utf-8')

                postnames.append(str(filename))

            counter += 1

            self.log.info("Posting files: %s", postnames) 

            self.news_poster.post(postnames)
            remove_file(postnames)


if __name__ == "__main__":
    PostData(sys.argv).run()




from datatransport.utilities import size_desc 

def test_sizedesc_metric_b():
    value = 200 
    result = size_desc(value, metric=True) 
    assert result == '200.0 B' 

def test_sizedesc_kb():
    value = 2000 
    result = size_desc(value, metric=True) 
    assert result == '2.0 kB' 

def test_sizedesc_mb():
    value = 2000000 
    result = size_desc(value, metric=True) 
    assert result == '2.0 MB' 

def test_sizedesc_gb():
    value = 2000000000 
    result = size_desc(value, metric=True) 
    assert result == '2.0 GB' 

def test_sizedesc_tb():
    value = 2000000000000
    result = size_desc(value, metric=True) 
    assert result == '2.0 TB' 

def test_sizedesc_kib():
    value = 2000 
    result = size_desc(value) 
    assert result == '2.0 KiB' 

def test_sizedesc_mib():
    value = 2000000 
    result = size_desc(value) 
    assert result == '1.9 MiB' 

def test_sizedesc_gib():
    value = 2000000000 
    result = size_desc(value) 
    assert result == '1.9 GiB' 

def test_sizedesc_tib():
    value = 2000000000000
    result = size_desc(value) 
    assert result == '1.8 TiB' 

def test_sizedesc_precision_2():
    value = 1234 
    result = size_desc(value, metric=True, precision=2) 
    assert result == '1.23 kB' 

def test_sizedesc_precision_3():
    value = 1234 
    result = size_desc(value, metric=True, precision=3) 
    assert result == '1.234 kB' 

def test_sizedesc_boundary():
    value = 1023.95 
    result = size_desc(value, metric=True, precision=0) 
    assert result == '1 kB' 

def test_sizedesc_negative():
    value = -600 
    result = size_desc(value, metric=True, precision=0) 
    assert result == '-600 B' 






#!/usr/bin/env python3

##########################################################################
#
#   Create initial data transport application directory layout.
#
#   2020-10-09  Todd Valentic
#               Initial implementation
#
##########################################################################

import argparse
import grp
import logging
import os
import platform
import pwd
import shutil
import sys

from pathlib import Path

if sys.version_info < (3,10):
    import importlib_resources as resources
else:
    from importlib import resources

import datatransport

def parse_args(args=sys.argv[1:]):

    user = pwd.getpwuid(os.getuid()).pw_name
    group = grp.getgrgid(os.getgid()).gr_name
    hostname = platform.node().split('.',1)[0]

    parser = argparse.ArgumentParser(
                description='Create a new data transport application'
                )

    parser.add_argument(
        '-d', '--dir',
        help='Installation directory',
        default='.'
        )

    parser.add_argument(
        '-u', '--user',
        help=f'Run as user "{user}"',
        default=user
        )

    parser.add_argument(
        '-g', '--group',
        help=f'Run as group "{group}"',
        default=group
        )

    parser.add_argument(
        '-n', '--hostname',
        help=f'Hostname "{hostname}"',
        default=hostname
        )

    parser.add_argument(
        '-v', '--verbose',
        help='Verbose output',
        action='store_true'
        )
    return parser.parse_args(args)

def create_dirs(basepath):

    basepath.mkdir(exist_ok=True, parents=True)

    dirs = ['bin', 'etc', 'var', 'tmp', 'log', 'groups']

    for leaf in dirs:
        basepath.joinpath(leaf).mkdir(exist_ok=True)

    basepath.joinpath('groups').chmod(0o2775)


def install_config(args, basepath):

    for entry in resources.files('datatransport.templates.etc').iterdir():
        contents = entry.read_text()
        contents = contents.replace('@PREFIX@', str(basepath.absolute()))
        contents = contents.replace('@PACKAGE_VERSION@', datatransport.__version__)
        contents = contents.replace('@USER@', args.user)
        contents = contents.replace('@GROUP@', args.group)
        contents = contents.replace('@HOSTNAME@', args.hostname)

        outputPath = basepath / 'etc' / entry.name
        
        with outputPath.open(mode='w') as output:
            output.write(contents)

def link_programs(package, basepath):

    venv_path = Path(os.environ["VIRTUAL_ENV"],"bin")
    dest_path = basepath.joinpath('bin')

    programs = [
        "cancelnewsgroup",
        "console",
        "listnewsgroups",
        "transport-create-app",
        "transportd",
        "transportps",
        "viewlog"
    ]

    for program in programs:
        src = venv_path.joinpath(program)
        dst = dest_path.joinpath(program)
        dst.symlink_to(src)

def install_groups(package, basepath):

    basepath.mkdir(exist_ok=True)
    basepath.chmod(0o2775)

    for entry in [r.name for r in resources.files(package).iterdir()]:
        if resources.files(package).joinpath(entry).is_file():
            with resources.path(package, entry) as srcpath:
                destpath = basepath / entry
                shutil.copyfile(srcpath, destpath) 
                destpath.chmod(0o775)
                logging.debug(f'Copy f{srcpath} -> {basepath}')
        else:
            install_groups(f'{package}.{entry}', basepath / entry)

def main():

    args = parse_args()

    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)

    basepath = Path(args.dir)

    create_dirs(basepath)
    install_config(args, basepath)
    link_programs(args, basepath)
    install_groups('datatransport.templates.groups', basepath / 'groups')

    print(f'Created in {basepath}')




#!/usr/bin/env python
"""Test confif parser get_bytes"""

##########################################################################
#
#   Test get_bytes config settings
#
#   2022-11-02  Todd Valentic
#               Initial implementation
#
##########################################################################

import sys

from datatransport import ProcessClient


class Demo(ProcessClient):
    """Test program"""

    def __init__(self, args):
        ProcessClient.__init__(self, args)

        for key in self.options():
            if key.startswith('test.'):
                try:
                    self.log.info("%s: %s", key, self.config.get_bytes(key))
                except Exception as e:  # pylint: disable=broad-exception-caught
                    self.log.error('Problem parsing %s: %s', key, e)

if __name__ == "__main__":
    Demo(sys.argv).run()



#!/usr/bin/env python3
"""WatchURL Component"""

#####################################################################
#
#   WatchURL
#
#   This script watches a URL (which, for example, can be a web
#   page or an image) and posts it to the specified newsgroup when
#   a new version appears.
#
#   Based on code written by Angela Li (SRI International).
#
#   History:
#
#   1.0.0   2002-01-15  TAV
#           Initial implementation.
#
#   1.0.1   2002-09-03  Todd Valentic
#           Moved to python2, sri.transport -> Transport
#           Use python2 md5's hexdigest() (replace local hexstr()).
#           Added retry time for problem sites
#           Added header checks, exclude list
#           Added indirect url features.
#
#   1.0.2   2002-10-13  Todd Valentic
#           Added ability to create thumbnails. These are posted
#               as another attachment in the message. Makes it
#               nice when used with ArchiveGroups and the history
#               function.
#
#   1.0.3   2003-03-02  Todd Valentic
#           Added ability to sync the pollrate.
#
#   1.0.4   2003-03-08  Todd Valentic
#           Put the news post in try..except block to catch errors.
#
#   1.0.5   2003-03-31  Todd Valentic
#           Added offset to compliment sync.
#
#   1.0.6   2003-04-16  Todd Valentic
#           Added use of timeoutsocket
#
#   1.0.7   2003-08-25  Todd Valentic
#           Use DeltaDateTime objects in config
#           Added ability to exclude image names in patterns check
#           Added images in anchor tags to those checked
#           Added small test for parser at end of file
#
#   1.0.8   2005-08-15  Todd Valentic
#           Change sync to be a boolean.
#
#   1.0.8   2009-05-11  Todd Valentic
#           md5 module deprecated in Python 2.6. Use a import check
#               to pull in right library on older systems.
#
#   1.0.8   2013-01-29  Todd Valentic
#           Add timeout parameter
#
#   1.0.9   2013-08-18  Todd Valentic
#           Add time replacement to rename rules
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#               Python3 migration:
#                   commands -> subprocess
#                   <> -> !=
#                   htmllib -> html.parser
#                   urlparse -> urllib.parse
#                   removeFile -> remove_file
#                   NewsPoster
#
#   2023-07-26  Todd Valentic
#               Updated for transport3 / python3
#
#####################################################################

import fnmatch
import hashlib
import nntplib
import os
import pathlib
import socket
import subprocess
import sys
import urllib.request
import urllib.error

from html.parser import HTMLParser
from urllib.parse import urlparse, urljoin

from datatransport import ProcessClient
from datatransport import NewsPoster
from datatransport.utilities import remove_file


class Parser(HTMLParser):
    """HTML Parser"""

    def __init__(self, include_names, exclude_names):
        HTMLParser.__init__(self)

        self.include_patters = include_names
        self.exclude_patterns = exclude_names

    def get_images(self, url):
        """Get image URLS on page"""

        self.urls = []
        self.base = url

        with urllib.request.urlopen(url) as response:
            contents = response.read()

        self.reset()
        self.feed(contents.decode("utf-8"))
        self.close()

        return self.urls

    def match(self, url):
        """Check of URL matches pattern"""

        path = urlparse(url).path

        for pattern in self.exclude_patterns:
            if fnmatch.fnmatch(path, pattern):
                return False

        for pattern in self.include_patters:
            if fnmatch.fnmatch(path, pattern):
                return True

        return False

    def handle_starttag(self, tag, attrs):
        if tag == "img":
            self.handle_image(dict(attrs))

        elif tag == "a":
            self.handle_anchor(dict(attrs))

    def handle_image(self, attrs):
        """Handle image tag"""

        src = attrs["src"]

        if self.match(src):
            self.urls.append(urljoin(self.base, src))

    def handle_anchor(self, attrs):
        """Handle anchor tag"""

        href = attrs["href"]

        if self.match(href):
            self.urls.append(urljoin(self.base, href))


class WatchURL(ProcessClient):
    """Process Client"""

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        self.news_poster = NewsPoster(self)

        self.pollrate = self.config.get_rate("pollrate", "10m")
        self.retryrate = self.config.get_rate("retryrate", 60)
        self.timeout = self.config.get_timedelta("timeout", 60)

        self.url = self.config.get("url", None)
        self.include_names = self.config.get_list("images", "*")
        self.exclude_names = self.config.get_list("images.exclude", "")
        self.check_headers = self.config.get_boolean("headers.enable", True)
        self.exclude_headers = self.config.get_list("headers.exclude", "Date")
        self.rename_rules = self.config.get("rename", "[]")
        self.save_body = self.config.get_boolean("save.body", True)
        self.save_images = self.config.get_boolean("save.images", False)
        self.save_thumbnails = self.config.get_boolean("save.thumbnails", False)
        self.thumbnail_cmd = self.config.get(
            "thumbnails.cmd", "convert -geometry 125x125 -sharpen 2x2"
        )
        self.thumbnail_exts = self.config.get_list("thumbnails.ext", ".jpg .png .gif")
        self.thumbnail_name = self.config.get("thumbnails.name", "%s-thumbnail.jpg")

        self.checksum_files = pathlib.Path("checksum.files")
        self.checksum_headers = pathlib.Path("checksum.headers")

        socket.setdefaulttimeout(self.timeout.total_seconds())

        try:
            self.rename_rules = eval(self.rename_rules)
        except:
            self.log.exception("Problem parsing the rename rules:")
            self.abort("Aborting")

        if self.url is None:
            self.abort("No URL specified")

        scheme = urlparse(self.url).scheme

        if self.save_images and not scheme.startswith("http"):
            self.abort("Can only save images from http sites")

        self.parser = Parser(self.include_names, self.exclude_names)

    def gather_urls(self, src_url):
        """Find URLs on the page"""

        urls = []

        if self.save_body:
            urls.append(src_url)

        if self.save_images:
            urls.extend(self.parser.get_images(src_url))

        self.log.debug("Checking these URLs:")
        for url in urls:
            self.log.debug("  %s", url)

        return urls

    def get_headers(self, url):
        """Retrieve headers"""

        self.log.debug("  getting headers from %s", url)

        with urllib.request.urlopen(url) as response:
            headers = response.info()

        keepers = []

        for header, value in headers.items():
            if header not in self.exclude_headers:
                self.log.debug("    including %s", header)
                keepers.append(f"{header}: {value}")
            else:
                self.log.debug("    excluding %s", header)

        return "\n".join(keepers).encode("utf-8")

    def headers_changed(self, urls):
        """Check if headers have changed"""

        self.log.debug("Checking headers")

        try:
            prev_checksum = self.checksum_headers.read_text("utf-8")
        except OSError:
            prev_checksum = 0

        checksum = hashlib.md5()

        for url in urls:
            checksum.update(self.get_headers(url))

        checksum = checksum.hexdigest()

        self.log.debug("  prev checksum: %s", prev_checksum)
        self.log.debug("  new  checksum: %s", checksum)

        if checksum != prev_checksum:
            self.log.debug("  header change detected")
            self.checksum_headers.write_text(checksum, "utf-8")
            return True

        return False

    def files_changed(self, checksum):
        """Check if files have changed"""

        self.log.debug("Checking file checksum")

        try:
            prev_checksum = self.checksum_files.read_text("utf-8")
        except OSError:
            prev_checksum = 0

        self.log.debug("  prev checksum: %s", prev_checksum)
        self.log.debug("  new  checksum: %s", checksum)

        if checksum != prev_checksum:
            self.log.debug("  new files detected")
            self.checksum_files.write_text(checksum, "utf-8")
            return True

        self.log.debug("  files are the same")
        return False

    def remap(self, filename):
        """Rename file"""

        for pattern, result in self.rename_rules:
            if fnmatch.fnmatch(filename, pattern):
                return self.now().strftime(result)

        return filename

    def retrieve_files(self, urls):
        """Retrieve files"""

        filenames = []
        unknownext = 0
        checksum = hashlib.md5()

        self.log.debug("Retrieving files:")

        for url in urls:
            path = urlparse(url).path
            filename = os.path.basename(path)
            if not filename:
                filename = f"unknown.{unknownext}"
                unknownext = unknownext + 1

            filename = self.remap(filename)

            try:
                with urllib.request.urlopen(url) as response:
                    contents = response.read()
                checksum.update(contents)
                with open(filename, "wb") as f:
                    f.write(contents)
                filenames.append(filename)
                self.log.debug("  %s -> %s", url, filename)
            except Exception as err: # pylint: disable: broad-exception-caught
                self.log.error("Problem retrieving %s: %s", url, err)

        return filenames, checksum.hexdigest()

    def make_thumbnails(self, filenames):
        """Create thumbnails"""

        self.log.debug("Making thumbnails")

        thumbnails = []

        for filename in filenames:
            root, ext = os.path.splitext(filename)
            if ext in self.thumbnail_exts:
                self.log.debug("  creating thumbnail for %s", filename)
                outputname = self.thumbnail_name % root
                cmd = f"{self.thumbnail_cmd} {filename} {outputname}"
                status, output = subprocess.getstatusoutput(cmd)

                if status == 0:
                    thumbnails.append(outputname)
                else:
                    self.log.error("Problem running thumbnail command: %s", cmd)
                    self.log.error("Output: %s", output)

        return thumbnails

    def main(self):
        self.wait(self.pollrate)

        while self.is_running():
            try:
                urls = self.gather_urls(self.url)
            except:
                self.log.exception("Problem gathering URLs")
                self.wait(self.retryrate)
                continue

            if not urls:
                self.log.debug("No urls to check")
                self.wait(self.pollrate)
                continue

            try:
                if self.check_headers and not self.headers_changed(urls):
                    self.log.debug("The headers have not changed")
                    self.wait(self.pollrate)
                    continue
            except:
                self.log.exception("Problem checking headers")
                self.wait(self.retryrate)
                continue

            filenames, checksum = self.retrieve_files(urls)

            if self.files_changed(checksum):
                if self.save_thumbnails:
                    try:
                        filenames.extend(self.make_thumbnails(filenames))
                    except:
                        self.log.exception("Problem making thumbnails")

                if filenames:
                    try:
                        self.news_poster.post(filenames)
                        self.log.info("New files detected, posting %s", filenames)
                    except nntplib.NNTPError as err:
                        self.log.error("Error posting to the news server: %s", err)

            remove_file(filenames)

            self.wait(self.pollrate)


def main():
    """Script entry point"""
    WatchURL(sys.argv).run()



#!/usr/bin/env python
"""Test Config Components"""

##############################################################
#
#   ConfigComponents Demonstration
#
#   2004-02-01  Todd Valentic
#               Initial implementation.
#
#   2022-11-04  Todd Valentic
#               Updated for Python3
#
##############################################################

import sys

from datatransport import ProcessClient
from datatransport import ConfigComponent
from datatransport import NewsPoster


class Watch(ConfigComponent):
    """Component Class"""

    def __init__(self, name, config, parent, **kw):
        ConfigComponent.__init__(self, "watch", name, config, parent, **kw)

        self.news_poster = NewsPoster(self)

        self.path = self.config.get_path("path", ".")
        self.label = self.config.get("label", "")
        self.host = self.config.get("host", "")
        self.option = self.config.get("option", "")
        self.desc = self.config.get("desc", "")
        self.subject = self.config.get("subject", "")

        self.log.info("Component: %s", self.name)
        self.log.info("  path:    %s", self.path)
        self.log.info("  label:   %s", self.label)
        self.log.info("  host:    %s", self.host)
        self.log.info("  option:  %s", self.option)
        self.log.info("  desc:    %s", self.desc)
        self.log.info("  subject: %s", self.subject)
        self.log.info("  options: %s", self.config.options())


class Demo(ProcessClient):
    """Parent Class"""

    def __init__(self, args):
        ProcessClient.__init__(self, args)

        self.components = self.config.get_components("watches", factory=Watch)

        self.log.info("Loaded %d components:", len(self.components))

        for name in self.components:
            self.log.info("  %s", name)

if __name__ == "__main__":
    Demo(sys.argv).run()



#!/usr/bin/env python3

from datatransport.apps import SchedulerEvent 

class PanoramaEvent(SchedulerEvent):

    def __init__(self, *p, **kw):
        SchedulerEvent.__init__(self, *p, **kw)

        self.label = self.config.get("label")

    def run(self, endtime):
        self.log.info("Run: %s %s", self.label, endtime)




#!/usr/bin/env python3
"""Demonstrate an XMLRPC Service"""

##########################################################################
#
#   Echo Service
#
#   Example service that simply echos back a message.
#
#   2022-11-02  Todd Valentic
#               Initial implementation
#
##########################################################################

import sys

from datatransport import ProcessClient
from datatransport import XMLRPCServer

from socketserver import ThreadingMixIn

class ThreadedServer(ThreadingMixIn, XMLRPCServer):
    pass

class EchoService(ProcessClient):
    """Echo Service"""

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        if self.config.get_boolean('threaded', False):
            self.xmlserver = ThreadedServer(self) 
        else:
            self.xmlserver = XMLRPCServer(self)

        self.main = self.xmlserver.main

        self.xmlserver.register_function(self.emit)

    def emit(self, msg):
        """Return message to caller"""

        self.log.info("Start emit message '%s'", msg)

        self.wait(10)

        self.log.info("Stop emit message '%s'", msg)

        return msg


if __name__ == "__main__":
    EchoService(sys.argv).run()



#!/usr/bin/env python
"""Transport Manager"""

##########################################################################
#
#   Create and run a transport server instance.
#
#   2022-10-07  Todd Valentic
#               Add header comment block
#               Reorder imports
#
##########################################################################

import os
import queue
import signal
import subprocess
import time

from . import TransportServer


def handler(_signum, _frame):
    """Signal handler"""
    os.wait()


class TransportManager:
    """Create and run a TransportServer"""

    def __init__(self):
        signal.signal(signal.SIGINT, self.handler)
        signal.signal(signal.SIGTERM, self.handler)
        signal.signal(signal.SIGHUP, self.handler)
        signal.signal(signal.SIGCHLD, handler)

        self.queue = queue.Queue()

        self.tasks = []
        self.server = TransportServer(self.queue)

    def handler(self, _signum, _frame):
        """Handle stop signals"""

        self.server.stop()

    def launch(self, args, environ):
        """Start running a client process"""

        return subprocess.Popen(
            args, env=environ, stderr=subprocess.PIPE, bufsize=1, text=True
        )

    def check(self, tasks):
        """Check the status of launched tasks"""

        running = []

        for task in tasks:
            if task.poll() is None:
                running.append(task)
                continue

            _stdout, stderr = task.communicate()

            if stderr:
                self.server.log.error("Problem with %s:", ' '.join(task.args))

                for line in stderr.split("\n"):
                    self.server.log.error(line)

        return running

    def run(self):
        """Main loop"""

        self.server.start()

        delay = self.server.config.get_timedelta("client.delay", 0.5)
        tasks = []

        while self.server.running:
            tasks = self.check(tasks)

            try:
                _cmd, args, environ = self.queue.get(timeout=0.5)
            except queue.Empty:
                continue

            try:
                # --pylint: disable=consider-using-with
                task = self.launch(args, environ)
            except (PermissionError, subprocess.SubprocessError) as err:
                self.server.log.error("Problem starting %s: %s", args, err)

            time.sleep(delay.total_seconds())
            tasks.append(task)

        self.server.shutdown()
        self.server.join()

        return 0



#!/usr/bin/env python
"""Access Mixin"""

#########################################################################
#
#   Access Mixin
#
#   This mixin provides access to a parent ProcessClient's
#   methods for configuration lookup and logging. It is used
#   by companion classes (usually running in a separate
#   thread). It is similar to ConfigComponent.
#
#   2005-11-12  Todd Valentic
#               Initial implementation.
#
#   2006-10-26  Todd Valentic
#               Added currentTime and utc
#
#   2020-01-16  Todd Valentic
#               Added isRunning, isStopped
#
#   2022-10-07  Todd Valentic
#               Update imports
#               currentTime -> current_time
#
#   2023-07-26  Todd Valentic
#               Updated for transport3 / python3
#               Use Root for base class 
#
#   SPDX-License-Identifier: GPL-3.0-or-later
#   Copyright (C) 1999-2022 Todd Valentic
#
#########################################################################

from . import Root


class AccessMixin(Root):
    """Connect a child class to common methods in the parent"""

    def __init__(self, parent):

        if not hasattr(self, 'config'):
            self.config = parent.config

        self.parent = parent
        self.log = parent.log
        self.abort = parent.abort
        self.exit = parent.exit
        self.stop = parent.stop
        self.wait = parent.wait
        self.now = parent.now
        self.utc = parent.utc
        self.is_running = parent.is_running
        self.is_stopped = parent.is_stopped



#!/usr/bin/env python
"""Data Transport Service"""

###########################################################################
#
#   transportd
#
#   Start an instance of the TransportServer.
#
#   History:
#
#   1.0.0   2000-06-18  TAV
#           Initial implementation.
#           Separated startup code from TrasnportServer.py
#
#   2.0.0   2002-02-22  Todd Valentic
#           Modified daemon routine to not set the uid/gid. We are now
#               started by the transport user in the /etc/init.d/transportd
#               script. This sets the supplemental groups properly until
#               we are running under python2 (which has os.setgroups).
#               Once we are running python2, make this back into a proper
#               daemon setup.
#
#   2.0.1   2002-08-28  Todd Valentic
#           sri.transport -> Transport
#           Setup with configure
#
#   2.0.2   2002-09-04  Todd Valentic
#           Set the supplementary groups. With the change to python2, we
#               now can use os.setgroups() to set all of the groups that
#               the transport user is part of.
#           Set umask to 0 in daemon mode.
#
#   2.0.3   2003-07-01  Todd Valentic
#           Setting umask to 0 isn't exactly what we want. Now take it
#               as a parameter from the config file (default is 0002).
#
#   2.1.0   2004-08-02  Todd Valentic
#           Converted from CORBA to XML-RPC.
#
#   2.1.1   2005-01-06  Todd Valentic
#           Changed imports for TransportManager and TransportConfig
#
#   2016-12-23  Todd Valentic
#               Migrate to datatransport package format
#
#   2022-10-06  Todd Valentic
#               Move into datatransport/commands
#               Only change user/group if started as root
#
############################################################################

import argparse
import grp
import os
import pwd
import sys

from datatransport import TransportConfig
from datatransport import TransportManager


def initgroups(user, gid):
    """Initialize all of groups for which the user is a member"""

    db = grp.getgrall()

    groups = set(gid)

    for _name, _passwd, gr_gid, gr_mem in db:
        if user in gr_mem:
            groups.add(gr_gid)

    os.setgroups(list(groups))


def daemon(config):
    """Launch as a daemon"""

    os.chdir("/")

    if os.fork():
        os._exit(0)

    os.setsid()

    # pylint: disable=consider-using-with

    sys.stdin = sys.__stdin__ = open("/dev/null", "rb")
    sys.stdout = sys.__stdout__ = open("/dev/null", "wb")
    sys.stderr = sys.__stderr__ = os.dup(sys.stdout.fileno())

    os.umask(config.get_int("umask"))

    username = config.get("username")
    groupname = config.get("groupname")

    if os.getuid() == 0 and username:
        uid = pwd.getpwnam(username).pw_uid
        gid = pwd.getgrnam(groupname).gr_gid

        initgroups(username, gid)
        os.setgid(gid)
        os.setuid(uid)


def main():
    """Main application"""

    config = TransportConfig()["TransportServer"]
    parser = argparse.ArgumentParser(description="Data Transport Network server")

    parser.add_argument(
        "-b", "--daemon", action="store_true", help="Run in the background"
    )

    args = parser.parse_args()

    if args.daemon:
        daemon(config)

    pidfile = config.get_path("pid.file")
    pidfile = pidfile.absolute()
    pidfile.parent.mkdir(parents=True, exist_ok=True)
    pidfile.write_text(f"{os.getpid()}")

    manager = TransportManager()

    status = manager.run()

    pidfile.unlink(missing_ok=True)

    return status



#!/usr/bin/env python3
"""File posting example"""

##########################################################################
#
#   File post example
#
#   Periodically post a file to a news group
#
#   2022-10-10  Todd Valentic
#               Initial implementation
#
##########################################################################

import sys
import pathlib

from datatransport import ProcessClient
from datatransport import NewsPoster


class Client(ProcessClient):
    """Process Client"""

    def __init__(self, args):
        ProcessClient.__init__(self, args)

        self.news_poster = NewsPoster(self)

        self.rate = self.config.get_rate("rate")
        self.text = self.config.get("text")

        self.log.info("Post message every: %s", self.rate.period)

    def main(self):
        """Main application"""

        filename = pathlib.Path("data.txt")

        while self.wait(self.rate):
            filename.write_text(self.text, encoding="utf-8")

            self.news_poster.post(filename)
            self.log.info("Posted %s", filename)

            filename.unlink()


if __name__ == "__main__":
    Client(sys.argv).run()



#!/usr/bin/env python3
"""Post data files component"""

############################################################################
#
#   PostDataFiles
#
#   This script watches for new radar data files to appear and posts them
#   to the newsserver. There is an option not to remove the original file
#   under the assumption that someone else will in the future. It can also
#   compress the files prior to posting using bzip to reduce their size
#   (we get about a 30% reduction in size on binary MEDAC data).
#
#   The data files are assumed to have a name with the following format:
#
#       <index>.<ext>
#
#   The <index> is a monotonically increasing value that can be tracked to
#   determine if the file has been posted before. The <ext> can be any
#   arbitrary string. The algorithm for determing which new files to post
#   is based on the finding files in the specified directory and with the
#   specified name that are newer since the last time the poll was taken.
#   Of these new files, the index values are compared to the previously
#   posted file and only newer files are posted.
#
#   It is assumed that the newest file in the directory is still being
#   written to by the data aqcuisition system and shouldn't be posted.
#   You can override this behaviour with the "include_current" option.
#
#   1.0.0   2001-02-20  Todd Valentic
#           Initial implementation.
#
#   1.0.1   2001-11-30  Todd Valentic
#           Quote the the filename pattern in the poll() method.
#           Added some debugging message to verbose level 2.
#
#   1.0.2   2001-12-02  Todd Valentic
#           Only update the timestamp file as each source file is
#               processed. Update the time stamp not with the current
#               time, but the time of the source file.
#           Added signal handling to make sure we done a clean exit.
#
#   2.0.0   2002-01-18  Todd Valentic
#           Generalized from PostDataFiles to handle multiple types of
#               radar data.
#
#   2.0.1   2002-03-22  Todd Valentic
#           Remove signal handling (use ProcessClient's).
#           Use commands.getstatusoutput() in compress call.
#
#   2.0.2   2002-03-28  Todd Valentic
#           Change default value of startCurrent to false.
#
#   2.0.3   2002-08-15  Todd Valentic
#           Changed the comparison to handle generic string operations.
#               The filename does not need to be strictly a number anymore.
#           Made file compression optional. The conf option is "compress"
#               and defaults to true.
#           Added option to remove original file (removeFile). Default 0.
#           Added option to only post last N files found (default to all).
#
#   2.0.4   2002-08-27  Todd Valentic
#           Setup with configure script.
#           sri.transport -> Transport
#           split runtime into script and class module.
#
#   2.0.5   2003-04-16  Todd Valentic
#           Use string methods instead of module.
#
#   2.0.6   2003-04-18  Todd Valentic
#           Added ability to specify pollrate in days/hours/mins/secs.
#           Added sync option for wait.
#           If the timestamp file doesn't exist, initialize its time
#               to something really old (Jan 1, 1975). We start off
#               by looking for files newer then that file (unless
#               startCurrent==1). It used to be initialized to the
#               current time and old files never got pulled in when
#               first ran.
#
#   2.0.7   2003-05-02  Todd Valenic
#           Fixed bug in log reporting when a file had already been
#               processed (it was trying to print a string as an int).
#
#   2.0.8   2003-05-03  Todd Valentic
#           Added check_index to turn off index checking.
#
#   2.0.9   2003-06-20  Todd Valentic
#           Added filedate callback.
#
#   2.0.10  2003-08-14  Todd Valentic
#           createCallback -> get_callback
#           createDeltaTime -> getDeltaTime
#
#   2.0.11  2004-02-13  Todd Valentic
#           Skip compression if file is already compressed.
#           Add a maximum file size limit.
#
#   2.0.12  2004-05-07  Todd Valentic
#           Print input path/name to log at startup.
#
#   2.0.13  2004-05-28  Todd Valentic
#           Fixed bug when reporting compression ratio and the
#               input file is 0 length.
#
#   2.0.14  2004-12-27  Todd Valentic
#           Convert from mx.DateTime to datetime
#
#   2.0.15  2005-04-03  Todd Valentic
#           Use library functions to replace external calls to touch
#           Use library bz2 module
#
#   2.0.16  2005-05-04  Todd Valentic
#           Fix bug when timestamp file not present.
#           Changed a number of config parameters to booleans
#
#   2006-10-14  Todd Valentic
#               Convert headers in post to a dict from a list
#
#   2009-11-24  Todd Valentic
#               Use now()
#
#   2015-11-23  Todd Valentic
#               Add md5 checksum header.
#
#   2015-11-30  Todd Valentic
#               Use get_rate() for pollrate
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#               Use get_ config methods
#               Python3 updates:
#                   commands -> subprocess
#                   removeFile -> remove_file
#                   sizeDesc -> size_desc
#                   currentTime -> now
#                   NewsPoster
#
#   2023-07-26  Todd Valentic
#               Updated for transport3 / python3
#
############################################################################

import bz2
import hashlib
import math
import nntplib
import os
import shutil
import subprocess
import sys
import time

from pathlib import Path

from datatransport import ProcessClient
from datatransport import NewsPoster
from datatransport.utilities import size_desc


class PostDataFiles(ProcessClient):
    """Process Client"""

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        self.news_poster = NewsPoster(self)

        self.exit_on_failure = self.config.get_boolean("exit_on_error", False)

        self.input_name = self.config.get("input.name", "*.DAT")
        self.input_path = self.config.get("input.path", ".")
        self.include_current = self.config.get_boolean("include_current", False)
        self.compress = self.config.get_boolean("compress", True)
        self.remove_files = self.config.get_boolean("remove_files", False)
        self.max_files = self.config.get_int("max_files")
        self.max_size = self.config.get_bytes("max_size", "20Mb")
        self.check_index = self.config.get_boolean("check_index", True)
        self.check_size = self.config.get_boolean("check_size", True)

        self.pollrate = self.config.get_rate("pollrate", 60)
        self.filedate = self.config.get_callback("filedate", self.filedate_callback)

        self.timefile = Path("timestamp")
        self.indexfile = Path("index")

        if not self.timefile.exists():
            # Default to sometime long ago
            self.timefile.write_text("0", "utf-8")
            timestamp = time.mktime((1975, 1, 1, 0, 0, 0, 0, 0, 0))
            os.utime(self.timefile, (timestamp, timestamp))

        if self.config.get_boolean("start_current", False):
            os.utime(self.timefile, None)

        self.log.info("Input path: %s", self.input_path)
        self.log.info("Input name: %s", self.input_name)

    def filedate_callback(self, _filename):
        """Extract timestamp from file"""

        return None

    def split(self, filename):
        """Split file into chunks"""

        filesize = filename.stat().st_size

        if not self.check_size or filesize <= self.max_size:
            return [filename]

        num_chunks = math.ceil(filesize / self.max_size)
        chunks = []

        with filename.open("rb") as f:
            for chunk in range(num_chunks):
                data = f.read(self.max_size)
                chunkname = Path(f"chunk.{chunk:03d}")
                chunkname.write_bytes(data)
                chunks.append(chunkname)

            self.log.info("  - split into %d parts", num_chunks)

        return chunks

    def compute_checksum(self, filename):
        """Compute checksum"""

        with filename.open("rb") as f:
            digest = hashlib.file_digest(f, "md5")

        return digest.hexdigest()

    def post(self, filename, timestamp):
        """Post file to newsgroup"""

        filenames = self.split(filename)
        numparts = len(filenames)
        checksum = self.compute_checksum(filename)

        for part, filepart in enumerate(filenames):
            headers = {}
            if numparts > 1:
                headers["X-Transport-Part"] = f"{part}/{numparts}"
                headers["X-Transport-Filename"] = filename.stem

            # Note - checksum of the *entire* file, not part
            headers["X-Transport-md5"] = checksum

            filesize = filepart.stat().st_size
            self.log.debug("  - posting %s (%s)", filepart, size_desc(filesize))
            self.news_poster.post([filepart], date=timestamp, headers=headers)

            if numparts > 1:
                os.remove(filepart)

    def valid_index(self, filename):
        """Check file is larger than last index"""

        if not self.check_index:
            return True

        if not self.indexfile.exists():
            return True

        last_filename = self.indexfile.read_text("utf-8")

        if filename.stem > last_filename:
            return True

        self.log.info("  - %s already processed (%s)", filename.stem, last_filename)

        return False

    def process_file(self, filename):
        """Process file (compress, split, post)"""

        self.log.info("Processing %s", filename)

        # Check if we have seen this file before

        if not self.valid_index(filename):
            return

        # Get timestamp from file

        try:
            timestamp = self.filedate(filename)
            if timestamp:
                self.log.debug("  - file timestamp %s", timestamp)
        except Exception:  # pylint: disable=broad-exception-caught
            self.log.exception("Error calling routine to determine file timestamp")
            return

        # Compress the file

        bzipext = ".bz2"
        zipname = filename.with_suffix(filename.suffix + bzipext)

        if self.compress and filename.suffix != bzipext:
            starttime = self.now()

            self.log.debug("  - compressing file")
            with filename.open("rb") as infile:
                with bz2.BZ2File(zipname, "wb", compresslevel=9) as outfile:
                    shutil.copyfileobj(infile, outfile)

            orgsize = filename.stat().st_size
            zipsize = zipname.stat().st_size

            if orgsize > 0:
                zippct = (zipsize / orgsize) * 100
            else:
                zippct = 0

            totaltime = self.now() - starttime
            self.log.info(
                "  - %s -> %s (%d%%) %s",
                size_desc(orgsize),
                size_desc(zipsize),
                zippct,
                totaltime,
            )

            postfile = zipname

        else:
            postfile = filename

        # Post file to news server

        try:
            self.post(postfile, timestamp)
        except nntplib.NNTPError as err:
            self.log.error("Problem posting file: %s", err)
            return

        # Cleanup files

        if zipname.exists():
            zipname.unlink()

        if self.remove_files:
            try:
                filename.unlink()
                self.log.debug("  - removed original file: %s", filename)
            except OSError as err:
                self.log.error("Problem removing file %s: %s", filename, err)

        # Update the index file

        if self.check_index:
            self.indexfile.write_text(filename.stem, "utf-8")

    def poll(self):
        """Poll for new files"""

        # pylint: disable=consider-using-f-string

        cmd = 'find %s -newer %s -type f -name "%s" -print' % (
            self.input_path,
            self.timefile,
            self.input_name,
        )

        status, output = subprocess.getstatusoutput(cmd)

        if status != 0:
            self.log.debug("Problem running find")
            self.log.debug("cmd: %s", cmd)
            self.log.debug("status: %s", status)
            self.log.debug("output: %s", output)
            return []

        filelist = sorted(output.split("\n"))

        if not self.include_current:
            # exclude the current file
            filelist = filelist[:-1]

        if self.max_files:
            # only keep the last N files
            filelist = filelist[-self.max_files :]

        return [Path(filename) for filename in filelist]

    def process(self):
        """Post any new files"""

        filenames = self.poll()

        self.log.debug("Polling - found %d new files.", len(filenames))

        for filename in filenames:
            timestamp = filename.stat().st_mtime
            self.process_file(filename)
            os.utime(self.timefile, (timestamp, timestamp))

            if self.is_stopped():
                return

    def main(self):
        """Main application"""

        while self.wait(self.pollrate):
            try:
                self.process()
            except Exception as err:  # pylint: disable=broad-exception-caught
                if self.exit_on_error:
                    self.log.exception("Problem processing")
                    return
                self.log.error("Problem processing: %s", err)


def main():
    """Script entry point"""
    PostDataFiles(sys.argv).run()



##########################################################################
#
#   Cancel messages in a news group
#
#   2022-10-10  Todd Valentic
#               Add header comment block
#               Make a package command
#
##########################################################################

import argparse

from datatransport import newstool 

def main():
    
    parser = argparse.ArgumentParser()

    parser.add_argument('newsgroup')

    parser.add_argument('-s','--server',
                default='localhost',
                help='News server host (default %(default)s)')

    parser.add_argument('-p','--port',
                default=119, type=int,
                help='News server port (default %(default)s)') 

    args = parser.parse_args()

    server = newstool.NewsControl()
    server.set_server(args.server, port=args.port)

    if not server.has_newsgroup(args.newsgroup):
        print(f'Error: newsgroup does not exist: {args.newsgroup}')
        return 1 

    print(f'Canceling messages in {args.newsgroup}') 

    server.cancel_newsgroup(args.newsgroup)
    
    return 0



#!/usr/bin/env python
"""Test process spawning from client"""

############################################################################
#
#   Spawner
#
#   This script runs child process at specified intervals
#
#   History:
#
#   2002-12-19  Todd Valentic
#               Initial implementation
#
#   2022-11-04  Todd Valentic
#               Update for python3
#
############################################################################

import subprocess
import sys

from datatransport import ProcessClient


class Spawner(ProcessClient):
    """Spawn parent process"""

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

    def main(self):
        """Main loop"""

        rate = self.config.get_rate("spawn.rate", 10)
        cmd = self.config.get("spawn.command", "pwd")

        while self.wait(rate):

            self.log.info("Running command: %s", cmd)

            try:
                status, output = subprocess.getstatusoutput(cmd)
            except Exception: # pylint: disable=broad-exception-caught
                self.log.exception("Problem starting child")
                self.abort("Exiting")

            self.log.info("  Status=%d", status)
            self.log.info("  Output=%s", output)


if __name__ == "__main__":
    Spawner(sys.argv).run()



#!/usr/bin/env python3
"""Event Monitor Component"""

##########################################################################
#
#   Event Monitor
#
#   2009-03-14  Todd Valentic
#               Initial implementation.
#
#   2010-03-22  Todd Valentic
#               Allow member section name to be specified
#
#   2013-02-08  Todd Valentic
#               Make sure to see if section exists when
#                   loading compontent from saved state.
#
#   2016-12-23  Todd Valentic
#               Use datetime.timezone.utc
#
#   2022-10-19  Todd Valentic
#               Python3 port
#                   use get_ methods
#                   use sapphire.Parser
#
#   2023-07-27  Todd Valentic
#               Updated for transport3 / python3
#
##########################################################################

import os
import datetime

from dateutil import parser

from datatransport import ConfigComponent
import sapphire_config as sapphire

utc = datetime.timezone.utc


class State(ConfigComponent):
    def __init__(self, *pos, **kw):
        ConfigComponent.__init__(self, "state", *pos, **kw)

        self.starttime = None
        self.lasttime = None
        self.elapsed = None

    def mark(self, timestamp):
        self.starttime = None
        self.update(timestamp)

    def update(self, timestamp):
        if self.starttime is None:
            self.starttime = timestamp

        self.lasttime = timestamp
        self.elapsed = self.lasttime - self.starttime

    def save(self, config, section):
        config.set(section, self.itemname + "starttime", str(self.starttime))
        config.set(section, self.itemname + "lasttime", str(self.lasttime))
        config.set(section, self.itemname + "elapsed", str(self.elapsed))

    def load(self, config, section):
        self.starttime = self.get_datetime(config, section, self.itemname + "starttime")
        self.lasttime = self.get_datetime(config, section, self.itemname + "lasttime")
        try:
            self.elapsed = self.lasttime - self.starttime
        except:
            self.elapsed = None

    def get_datetime(self, config, section, key):
        try:
            timestr = config.get(section, key)
            return parser.parse(timestr).replace(tzinfo=utc)
        except:
            return None

    def history_entry(self):
        return "%s, %s, %s" % (self.name, self.starttime, self.lasttime)


class Member(ConfigComponent):
    def __init__(self, *pos, **kw):
        if "state_factory" in kw:
            self.state_factory = kw["state_factory"]
            del kw["state_factory"]
        else:
            self.state_factory = State
        ConfigComponent.__init__(self, *pos, **kw)

        self.states = self.get_components("states", self.state_factory)
        self.unknown = self.get("states.unknown", "unknown")
        self.max_history = self.get_int("maxhistory", 10)
        self.section = self.get("section", self.name)

        if self.unknown not in self.states:
            self.states[self.unknown] = self.state_factory(self.unknown, self)

        self.cur_state = self.unknown
        self.prev_state = self.unknown
        self.history = []
        self.max_history = self.max_history
        self.current = self.states[self.cur_state]
        self.previous = self.states[self.prev_state]
        self.changed = False

    def add_states(self, *pos):
        for name in pos:
            self.states[name] = self.state_factory(name, self)

    def update(self, state, timestamp=None):

        if timestamp is None:
            timestamp = self.now()

        self.current.update(timestamp)

        if self.cur_state == state:
            self.changed = False
        else:
            self.history.append(self.current.history_entry())
            if len(self.history) > self.max_history:
                self.history = self.history[-self.max_history :]

            self.changed = True
            self.prev_state = self.cur_state
            self.cur_state = state
            self.current = self.states[self.cur_state]
            self.previous = self.states[self.prev_state]

            self.current.mark(timestamp)

        return self.changed

    def save(self, config):

        if not config.has_section(self.section):
            config.add_section(self.section)

        for entry in self.states.values():
            entry.save(config, self.section)

        config.set(self.section, "cur_state", self.cur_state)
        config.set(self.section, "prev_state", self.prev_state)
        config.set(self.section, "changed", str(self.changed))
        config.set(self.section, "history", "\n".join(self.history))

    def load(self, config):

        for entry in self.states.values():
            entry.load(config, self.section)

        if self.section not in config.sections():
            config.add_section(self.section)
            config.set(self.section, "changed", "False")

        self.cur_state = config.get(self.section, "cur_state", self.unknown)
        self.prev_state = config.get(self.section, "prev_state", self.unknown)
        self.current = self.states[self.cur_state]
        self.previous = self.states[self.prev_state]
        self.changed = config.get_boolean(self.section, "changed", False)
        self.history = config.get(self.section, "history", "").splitlines()
        self.history = self.history[-self.max_history :]


class EventMonitor:
    def __init__(self, membersDict):
        self.members = membersDict

    def changed(self):
        return set([m.name for m in self.members.values() if m.changed])

    def save(self, filename, timestamp=None):

        if timestamp is None:
            timestamp = datetime.datetime.now(utc)

        data = sapphire.Parser()
        data.set("DEFAULT", "timestamp", str(timestamp))
        data.set("DEFAULT", "changed", " ".join(self.changed()))

        for member in self.members.values():
            member.save(data)

        tmpname = filename + ".tmp"
        output = open(tmpname, "w")
        data.write(output)
        output.flush()
        os.fsync(output.fileno())
        output.close()
        os.rename(tmpname, filename)

        return data

    def load(self, filename):

        data = sapphire.Parser()
        data.read(filename)

        for member in self.members.values():
            member.load(data)



#!/usr/bin/env python

############################################################################
#
#	Environ	
#
#	This script displays the environment variables is sees at startup. 
#
#	History:
#	
#	1.0.0 	2002-12-19	Todd Valentic	
#			Initial implementation
#
############################################################################

from 	Transport	import ProcessClient

import	sys
import	os

class Environ (ProcessClient):

	def __init__(self,argv):
		ProcessClient.__init__(self,argv)

		self.printEnviron()

	def printEnviron(self):

		self.log.info('Current Environment:')

		keys = os.environ.keys()
		keys.sort()

		for key in keys:
			self.log.info('%s = %s' % (key,os.environ[key]))

	def run(self):

		while self.wait(60): 
			pass

if __name__ == '__main__':
	Environ(sys.argv).run()





import pytest
import zoneinfo

from datetime import datetime, timedelta, timezone
from datatransport.utilities import PatternTemplate

def test_pattern_replace():
    replaceRule = PatternTemplate('rule')
    src = '<rule>'
    value = 'hello world'
    result = replaceRule(src, value)
    assert result == f'{value}' 

def test_pattern_no_replace():
    replaceRule = PatternTemplate('rule')
    src = '<otherrule>'
    value = 'hello world'
    result = replaceRule(src, value)
    assert result == '<otherrule>' 

def test_pattern_int_value():
    replaceRule = PatternTemplate('rule')
    src = '<rule>'
    value = 0 
    result = replaceRule(src, value)
    assert result == str(value) 

def test_pattern_multiple():
    replaceRule = PatternTemplate('rule')
    replaceName = PatternTemplate('name')
    src = '<rule>-<name>'
    rule_value = 'a'
    name_value = 'b'
    result = replaceRule(src, rule_value)
    result = replaceName(result, name_value)
    assert result == 'a-b'

def test_pattern_embedded():
    replaceRule = PatternTemplate('rule')
    src = 'Before <rule> after'
    value = 'hello world'
    result = replaceRule(src, value)
    assert result == f'Before {value} after'

def test_pattern_group():
    replaceGroup = PatternTemplate('group', sep='.')
    src = 'More complicated example: <group[0:2]> <group[-1]>'
    group = 'comp.lang.python'
    result = replaceGroup(src, group)
    assert result == 'More complicated example: comp.lang python'

def test_pattern_dict():
    replaceDict = PatternTemplate('header')
    src = 'Using a dictionary: <header["x-transport-date"]>'
    header = {'newsgroup': 'transport.data', 'x-transport-date': '2005-06-15'}
    result = replaceDict(src, header)
    assert result == 'Using a dictionary: 2005-06-15'

def test_pattern_cache():
    replaceDict = PatternTemplate('header')
    src = 'Using a dictionary: <header["x-transport-date"]>'
    header = {'newsgroup': 'transport.data', 'x-transport-date': '2005-06-15'}
    replaceDict.set_value(header)
    result = replaceDict(src)
    assert result == 'Using a dictionary: 2005-06-15'




#!/usr/bin/env python

############################################################################
#
#   transportctl
#
#   This script controls the transport server from the command line.
#
#   History:
#
#   1.0.0   1999.??.??  TAV
#           Initial implementation
#
#   1.2.0   2000.04.20  TAV
#           Updated to use omniORB for CORBA functions.
#
#   1.3.0   2000-06-22  TAV
#           Added ability to change options for the web-site users.
#
#   1.3.1   2000-08-08  TAV
#           Fixed bug if no option given on command line, call help.
#
#   1.3.2   2001-08-29  TAV
#           Handle new list of config files in the ProcessGroupInfo struct.
#
#   1.3.3   2001-10-23  TAV
#           Fixed a bug in the "user set" command. The user's values were
#               being reset to the defaults with each change.
#
#   1.3.4   2002-02-11  Todd Valentic
#           Added ability to pass args to start client command.
#
#   1.3.5   2002-08-28  Todd Valentic
#           Setup via configure script
#           sri.transport -> Transport
#           Removed web-site user functions.
#
#   1.3.6   2004-08-16  Todd Valentic
#           Cleanup for XML-RPC.
#           Removed log rotation.
#           Removed TransportClient.
#
#   2016-12-23  Todd Valentic
#               Migrate to datatransport package format
#
#   2020-10-09  Todd Valentic
#               Python3: xmlrpc.client
#               Python3: print
#
############################################################################

from datatransport import TransportConfig

import os
import shutil
import subprocess
import sys
import time
import xmlrpc.client

class TransportControl:

    def __init__(self):

        self.command = {    'help':     self.help,
                            'list':     self.list,
                            'status':   self.status,
                            'cleanup':  self.cleanup,
                            'reload':   self.reload,
                            'add':      self.addGroup,
                            'remove':   self.removeGroup,
                            'restart':  self.restart,
                            'start':    self.start,
                            'stop':     self.stop       }

    def help(self):

        print()
        print('-'*70)
        print('Transport Control Commands:')
        print('-'*70)
        print('\thelp                     - Show this page')
        print('\tstatus                   - Transport server status')
        print('\tloglevel                 - Set log level on group or server')
        print('\treload <group>           - Reload group\'s config file')
        print('\treload server            - Reload process groups')
        print('\tlist                     - List all process groups')
        print('\tlist <group>             - List clients in a group')
        print('\tstart <group>            - Start a process group')
        print('\tstop <group>             - Stop a process group')
        print('\trestart <group>          - Stop/start a process group')
        print('\tadd <group>              - Notify server of new group')
        print('\tremove <group>           - Remove a group from the server')
        print('\tstart server             - Start the transport server')
        print('\tstop server              - Stop the transport server')
        print('\tstart <group> <client>   - Start the client in group')
        print('\tstop <group> <client>    - Stop the client in group')
        print('\tcleanup                  - Remove processes after crash')
        print('-'*70)
        print()

    def runCommand(self,argv):

        if len(argv)<2:
            self.help()
            sys.exit(1)

        cmd = argv[1]
        arg = argv[2:]

        if cmd not in self.command:
            print("Unknown command:",cmd)
            self.help()
            return 1

        if cmd=='cleanup':
            self.cleanup()
            return 0
        elif cmd=='help':
            self.help()
            return

        url = TransportConfig().get('TransportServer','url')
        self.server = xmlrpc.client.ServerProxy(url)

        if not (cmd=='start' and len(arg)==1 and arg[0]=='server'):
            # Skip if we need to start the server

            try:
                self.server.status()
            except:
                print("Cannot connect to the transport server")
                return 1

        return self.command[cmd](arg)

    def printProcessGroup(self,group):
        print(group)
        return

        info = group._get_info()

        print('[%s]' % info.name)
        print('\tRunning      :',info.running)
        print('\tLabel        :',info.label)
        print('\tConfig files :',info.configFiles[0])
        for name in info.configFiles[1:]:
            print('\t             :',name)
        print('\tLog          :',info.log)
        print('\tStatus       :',info.status)

    def printProcessClient(self,client):
        print(client)
        return

        print('[%s]' % client.name)
        print('\tRunning      :',client.running)
        print('\tLabel        :',client.label)
        print('\tProcess ID   :',client.pid)
        print('\tStatus       :',client.status)

    def status(self,arg):
        print('Server status:',self.server.status())
        return 0

    def reload(self,arg):

        if arg==[]:
            print('You need to specify a process group or "server"')
            return 1

        if arg[0]=='server':
            self.server.reloadgroups()
            return 0

        group  = arg[0]

        try:
            self.server.reloadgroup(group)
        except:
            print('The process group "%s" does not exist' % group)
            print('  Use the "list" command to print the available groups')
            return 1

        return 0

    def addGroup(self,arg):

        if arg==[]:
            print('You need to specify a process group')
            return 1

        try:
            self.server.addgroup(arg[0])
        except:
            print('There was a problem adding the group "%s"' % arg[0])
            return 1

        return 0

    def removeGroup(self,arg):

        if arg==[]:
            print('You need to specify a process group')
            return 1

        try:
            self.server.removegroup(arg[0])
        except:
            print('There was a problem remove the group "%s"' % arg[0])
            return 1

        return 0

    def list(self,arg):
        if len(arg)==1:
            return self.listClients(arg[0])
        else:
            return self.listGroups()

    def listClients(self,group):
        groups = self.server.listgroups()

        if group not in groups:
            print('The process group "%s" does not exist' % group)
            print('  Use the "list" command to print the available groups')
            return 1

        clients = self.server.listclients(group)
        print('There are',len(clients),'clients listed for the group',group)
        for client,info in clients.items():
            #self.printProcessClient(client)
            print(client,info)
        return 0

    def listGroups(self):
        groups = self.server.listgroups()
        if not groups:
            print('No process groups are registered')
        else:
            print('There are',len(groups),'groups are registered:')
            for group,label in groups.items():
                print('%20s - %s' % (group,label))
                #self.printProcessGroup(group)
        return 0

    def startClient(self,group,client,args):

        groups = self.server.listgroups()

        if group not in groups:
            print('The process group "%s" does not exist' % group)
            print('  Use the "list" command to print the available groups')
            return 1

        try:
            self.server.startclient(group,client,args)
        except:
            print('The client "%s" does not exist' % client)
            return 1

        return 0

    def stopClient(self,group,client):

        groups = self.server.listgroups()

        if group not in groups:
            print('The process group "%s" does not exist' % group)
            print('  Use the "list" command to print the available groups')
            return 1

        try:
            self.server.stopclient(group,client)
        except:
            print('The client "%s" does not exsit' % client)
            return 1

        return 0

    def start_server(self):
        args = [shutil.which("transportd"), "-b"]
        subprocess.Popen(args)

    def start(self,arg):

        if len(arg)==0:
            print('You need to specify the group name or "server"')
            return 1

        if len(arg)==1 and arg[0]=='server':
            return self.start_server()

        if len(arg)>1:
            if len(arg)==3:
                args=arg[2]
            else:
                args=''
            return self.startClient(arg[0],arg[1],args)

        name = arg[0]

        try:
            self.server.startgroup(name)
        except:
            print('The process group "%s" does not exist' % name)
            print('  Use the "list" command to print the available groups')
            return 1

        print('Group started:',name)

        return 0

    def stop(self,arg):

        if len(arg)==0:
            print('You need to provide a group name or server')
            return 1

        elif len(arg)==2:
            self.stopClient(arg[0],arg[1])

        elif arg[0]=='server':
            self.server.stop()
            time.sleep(1)

        else:
            name = arg[0]

            try:
                self.server.stopgroup(name)
            except:
                print('The process group "%s" does not exist' % name)
                print('  Use the "list" command to print the available groups')
                return 1

            print('Group stopped:',name)

        return 0

    def restart(self,arg):
        self.stop(arg)
        self.start(arg)

    def cleanup(self):

        uid = TransportConfig().uid

        for file in os.listdir('/proc'):
            data = {}
            try:
                for line in open('/proc/%s/status' % file).readlines():
                    key,value = line.split(':')
                    data[key] = value.strip()
            except:
                continue

            if uid != int(data['Uid'].split()[0]):
                continue

            pid = int(data['Pid'])

            print('Stopping %5d - ' % pid,end=' ')
            try:
                os.kill(pid,9)
                print('OK')
            except:
                print('Could not kill process')

        print('Finished')

        return 0

def main():

    try:
        control = TransportControl()
        sys.exit(control.runCommand(sys.argv))
    except KeyboardInterrupt:
        print("*** BREAK ***")
        sys.exit(1)






from Transport.InstrumentStatus import Status

def callback(self,init=0,curstatus=None,prevstatus=None,changed=0):

	if init:
		self.log(1,'plugin init called')
		return

	if changed:
		self.log(1,'Callback called: change detected')




#!/usr/bin/env python
"""NewsPoster"""

##############################################################################
#
#   Data Transport NewsPoster
#
#   This class provides news posting functionality.
#
#   History:
#
#   1999-??-??  Todd Valentic
#               Initial implementation.
#
#   2000-04-26  Todd Valentic
#               Code cleanup, added enable checks, quiet flag.
#
#   2000-04-29  Todd Valentic
#               Now have the group creation part spin until group shows up.
#
#   2001-09-06  Todd Valentic
#               Added support for cross-posting. newsgroup can now be a
#                   list of groups.
#
#   2001-09-25  Todd Valentic
#               Set server default to be localhost.
#
#   2001-11-09  Todd Valentic
#               Renamed newstool -> NewsTool
#
#   2001-12-26  Todd Valentic
#               Changed tav -> sri
#
#   2002-01-24  Todd Valentic
#               Major changes.
#               Renamed config parameters so they use the prefix (default: post)
#               Server defualt is now localhost
#               Can now name the created object (default: newsPoster)
#
#   2002-03-22  Todd Valentic
#               Moved the mojority of the ctor code into createNewsPoster().
#                   This factory method can then be called multiple times and
#                   avoid the kludge idiom of calling the ctor again and again
#                   when more then one NewsPoster is needed.
#
#   2002-03-27  Todd Valentic
#               Fixed bug in reimplementation of createNewsPoster().
#
#   2002-08-28  Todd Valentic
#               sri.transport -> Transport
#
#   2003-04-15  Todd Valentic
#               Remove usage of string module - use string methods instead.
#               Cleaned up code createNewsPoster.
#
#   2004-02-05  Todd Valentic
#               Added enable config option.
#
#   2004-05-26  Todd Valentic
#               Change test for no news group.
#
#   2004-08-17  Todd Valentic
#               Changed to new logging interface.
#               Improved error reporting.
#
#   2005-01-06  Todd Valentic
#               Changed NewsTool import.
#
#   2005-03-06  Todd Valentic
#               Added creategroup option.
#
#   2005-06-13  Todd Valentic
#               Added ability to set extra post headers in config file.
#
#   2007-11-16  Todd Valentic
#               Make sure news group names are lower case
#
#   2008-07-18  Todd Valentic
#               Print news server and group names in error messages.
#
#   2008-08-11  Todd Valentic
#               Make sure we can connect to the news server at start.
#
#   2008-08-12  Todd Valentic
#               Correctly implement feature in 2.0.13.
#
#   2008-08-14  Todd Valentic
#               Fix another bug with 2.0.13 update. self.running not
#                   exposed in AccessMixin.
#
#   2008-08-18  Todd Valentic
#               Fix yet another (!) bug with 2.0.13 update. Only check
#                   news server connection if there are groups to create.
#
#   2009-05-05  Todd Valentic
#               Use setattr() instead of exec()
#
#   2009-05-11  Todd Valentic
#               Update to minor changes in NewsTool
#
#   2009-06-18  Todd Valentic
#               Fixed typo in createNewsPoster (post -> poster)
#
#   2009-07-23  Todd Valentic
#               Only check news server connection if we are creating
#                   groups (another 2.0.13 fix).
#
#   2012-10-05  Todd Valentic
#               Support setting server port
#
#   2021-07-04  Todd Valentic
#               Allow setting mulitple headers with .headers config
#
#   2022-10-06  Todd Valentic
#               Port to transport3 / python3
#                   newsPoster -> newsposter
#                   createNewsPoster -> postercreate
#
#   2023-07-04  Todd Valentic
#               Convert fom being a mixin class
#
#   2023-07-24  Todd Valentic
#               Ensure enable is a boolean
#               Use AccessMixin
#
##############################################################################

import fnmatch
import nntplib
import socket

from datatransport import AccessMixin
from datatransport import newstool


class NewsPoster(AccessMixin):
    """Data Transport NewsPoster"""

    def __init__(self, parent, **kwargs):
        self.poster = None
        AccessMixin.__init__(self, parent)

        self.poster = self.create_newsposter(**kwargs)

    def __getattr__(self, name):
        if self.poster:
            return getattr(self.poster, name)
        raise AttributeError()

    def get_headers(self, prefix):
        """Get extra headers from config file"""

        headers = {}

        for header in self.config.get_list(f"{prefix}.headers"):
            try:
                name, value = [v.strip() for v in header.split("=", 1)]
            except (AttributeError, ValueError):
                self.log.error(f"Failed to process header: {header}")
                continue

            headers[name.upper()] = value

        for key in fnmatch.filter(self.config.options(), f"{prefix}.header.*"):
            name = key.replace(f"{prefix}.header.", "").strip().upper()
            headers[name] = self.config.get(key).strip()

        if headers:
            self.log.info("Extra headers:")
            for name, value in headers.items():
                self.log.info(f"  {name}={value}")

        return headers

    def create_newsgroups(self, control, newsgroups):
        """Create newsgroup on server if missing"""

        host = f"{control.server_host}:{control.server_port}"

        control.open_server()

        for newsgroup in newsgroups:
            if not control.has_newsgroup(newsgroup):
                control.newgroup(newsgroup)
                self.log.info(f"Creating the post newsgroup {newsgroup} on {host}")

                while not control.has_newsgroup(newsgroup):
                    self.log.info("Waiting for newsgroup to show up")

                    if not self.wait(15):
                        self.abort("Exiting")

                self.log.info(f"Newsgroup {newsgroup} on {host} is ready")

    # pylint: disable=too-many-locals

    def create_newsposter(self, prefix="post", quiet=False):
        """Create a news poster"""

        host = self.config.get(f"{prefix}.newsserver", "localhost")
        port = self.config.get_int(f"{prefix}.newsserver.port", 119)
        newsgroups = self.config.get_list(f"{prefix}.newsgroup")
        enable = self.config.get_boolean(f"{prefix}.enable", True)
        creategroup = self.config.get_boolean(f"{prefix}.creategroup", True)

        headers = self.get_headers(prefix)

        newsgroups = [name.strip().lower() for name in newsgroups]

        if not newsgroups and not quiet:
            self.log.warning(
                f"No posting news group specified (expected {prefix}.newsgroup)"
            )

        from_header = self.config.get(f"{prefix}.from", "transport@localhost.net")
        subject_header = self.config.get(f"{prefix}.subject", "Unknown subject")

        poster = newstool.NewsPoster()
        poster.set_server(host, port)
        poster.set_newsgroup(",".join(newsgroups))
        poster.set_enable(enable and len(newsgroups) > 0)
        poster.set_from(from_header)
        poster.set_subject(subject_header)
        poster.set_log(self.log)

        for key, value in headers.items():
            poster.set_header(key, value)

        # Create the group if we need to

        control = newstool.NewsControl()
        control.set_server(host, port)

        if creategroup:
            while self.is_running():
                try:
                    self.create_newsgroups(control, newsgroups)
                    break
                except socket.error as e:
                    self.abort(f"Error connecting to news server: {host}:{port} {e}")
                except nntplib.NNTPError as e:
                    self.abort(f"Error creating news group: {host}:{port} {e}")
                except:  # pylint: disable=bare-except
                    self.log.exception("Error creating group:")
                    self.abort("Cannot create the posting newsgroup")

                self.wait(15)

        return poster



#####################################################################
#
#   List news groups on a news server
#
#   2012-09-24  Todd Valentic
#               Initial implementation
#
#   2022-10-07  Todd Valentic
#               Python3 updates
#                   optparse -> argparse
#
#####################################################################

import argparse
import nntplib
import sys

def lister(args):

    server = nntplib.NNTP(args.newsserver)
    groups = server.list()[1]

    counts = {}

    for group in groups:
        name  = group[0]
        first = int(group[2])
        last  = int(group[1])
        counts[name] = last-first+1

    if args.sort=='name':
        key = None
    else:
        key = counts.__getitem__

    for group in sorted(counts,key=key,reverse=args.reverse):
        line = ''
        if args.showcount:
            line += '%10d ' % counts[group]
        line += group

        print(line)

def main():

    parser = argparse.ArgumentParser()

    parser.add_argument('-n','--newsserver',
                      default='127.0.0.1',
                      help='News server address (default is %(default)s)')

    parser.add_argument('-c','--showcount',
                      action='store_true',
                      help='Show message counts')

    parser.add_argument('-s','--sort',
                     default='name',
                     choices=['name','count'],
                     help='Sort') 

    parser.add_argument('-r','--reverse',
                      action='store_true',
                      help='Reverse sort order')

    args = parser.parse_args()

    lister(args)

    return 0




#!/usr/bin/env python3

##########################################################################
#
#   MessageBox polling example
#
#   Periodically poll for messages from a news group 
#
#   2023-02-23  Todd Valentic
#               Initial implementation
#
##########################################################################

from datatransport import ProcessClient
from datatransport import MessageBoxPoller 

import sys

class Client (ProcessClient):

    def __init__(self, args):
        ProcessClient.__init__(self, args)

        self.poller = MessageBoxPoller(self)

        self.log.info(f'Poll for messages')

    def process(self, message):

        text = message.get_payload()
        self.log.info(f'Received "{text}"')

    def run(self):

        self.log.info('Start')
        
        self.poller.run(self.process)

        self.log.info('Stop')

if __name__ == '__main__':
    Client(sys.argv).run()




#!/usr/bin/env python
"""Test log rotation"""

############################################################################
#
#   Counter
#
#   Write something to the log file
#
#   History:
#
#   2004-02-08  Todd Valentic
#               Initial implementation
#
#   2022-11-04  Todd Valentic
#               Update for Python3
#
############################################################################

import sys

from datatransport import ProcessClient


class Counter(ProcessClient):
    """Test program"""

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

    def main(self):
        """Main loop"""

        rate = self.config.get_rate("rate", 1)
        counter = 0

        while self.wait(rate):
            self.log.info("Counter = %d", counter)
            counter += 1


if __name__ == "__main__":
    Counter(sys.argv).run()



#!/usr/bin/env python2

############################################################################
#
#   Crasher 
#
#   This script runs for awhile then dies to test the watchdog.
#   Assumes that the ServerMonitor/Watchdog is running.
#
#   History:
#   
#   1.0.0   2002-12-16  Todd Valentic   
#           Initial implementation
#
#   1.0.1   2005-01-09  Todd Valentic
#           Use DeltaTime for lifetime.
#
############################################################################

from    Transport   import ProcessClient

import  sys

class Crasher(ProcessClient):

    def __init__(self,argv):
        ProcessClient.__init__(self,argv)

    def run(self):

        lifetime = self.getDeltaTime('lifetime',10)

        while self.wait(lifetime): 
            self.log.info('About to crash!')
            badness = 1/0

if __name__ == '__main__':
    Crasher(sys.argv).run()

